/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package com.cloudgarden.resource;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import net.za.grasser.duplicate.gui.PercentageImage;
import net.za.grasser.duplicate.util.Chars;
import net.za.grasser.duplicate.util.res.Debug;

import org.apache.log4j.Logger;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Resource;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Widget;

/**
 * Class to manage SWT resources (Font, Color, Image and Cursor) There are no restrictions on the use of this code. You may change this code and your changes
 * will not be overwritten, but if you change the version number below then this class will be completely overwritten by Jigloo.
 * #SWTResourceManager:version4.0.0#
 */
public class SWTResourceManager {
  /**
   * <code>CURSOR</code> SWTResourceManager -
   */
  private static final String CURSOR = "CURSOR:"; //$NON-NLS-1$
  /**
   * <code>COLOR</code> SWTResourceManager -
   */
  private static final String COLOR = "COLOR:"; //$NON-NLS-1$
  /**
   * <code>LF_UNDERLINE</code> SWTResourceManager -
   */
  private static final String LF_UNDERLINE = "lfUnderline"; //$NON-NLS-1$
  /**
   * <code>LF_STRIKE_OUT</code> SWTResourceManager -
   */
  private static final String LF_STRIKE_OUT = "lfStrikeOut"; //$NON-NLS-1$
  /**
   * <code>DATA</code> SWTResourceManager -
   */
  private static final String DATA = "data"; //$NON-NLS-1$
  /**
   * <code>ORG_ECLIPSE_SWT_INTERNAL_WIN32_LOGFONT</code> SWTResourceManager -
   */
  private static final String ORG_ECLIPSE_SWT_INTERNAL_WIN32_LOGFONT = "org.eclipse.swt.internal.win32.LOGFONT"; //$NON-NLS-1$
  /**
   * <code>log</code> SWTResourceManager -
   */
  private static final Logger log = Logger.getLogger(SWTResourceManager.class);
  /**
   * <code>resources</code> SWTResourceManager -
   */
  private static final HashMap<String, Resource> resources = new HashMap<String, Resource>();
  /**
   * <code>users</code> SWTResourceManager -
   */
  static final Vector<Widget> users = new Vector<Widget>();
  /**
   * <code>instance</code> SWTResourceManager -
   */
  private static final SWTResourceManager instance = new SWTResourceManager();
  /**
   * <code>disposeListener</code> SWTResourceManager -
   */
  private static final DisposeListener disposeListener = new DisposeListener() {
    public void widgetDisposed(DisposeEvent e) {
      users.remove(e.getSource());
      if (users.size() == 0) {
        dispose();
      }
    }
  };

  /**
   * This method should be called by *all* Widgets which use resources provided by this SWTResourceManager. When widgets are disposed, they are removed from the
   * "users" Vector, and when no more registered Widgets are left, all resources are disposed.
   * <P>
   * If this method is not called for all Widgets then it should not be called at all, and the "dispose" method should be explicitly called after all resources
   * are no longer being used.
   * 
   * @param widget
   */
  public static void registerResourceUser(final Widget widget) {
    if (users.contains(widget)) {
      return;
    }
    users.add(widget);
    widget.addDisposeListener(disposeListener);
  }

  /**
   * This method disposes all resources in one go
   */
  public static void dispose() {
    for (final String key : resources.keySet()) {
      final Object resource = resources.get(key);
      if (resource instanceof Font) {
        ((Font)resource).dispose();
      } else if (resource instanceof Color) {
        ((Color)resource).dispose();
      } else if (resource instanceof Image) {
        ((Image)resource).dispose();
      } else if (resource instanceof Cursor) {
        ((Cursor)resource).dispose();
      }
    }
    resources.clear();
  }

  /**
   * This method gets a <code>Font</code> with the specified size and style.
   * 
   * @param name
   * @param size
   * @param style
   * @return Font
   */
  public static Font getFont(final String name, final int size, final int style) {
    return getFont(name, size, style, false, false);
  }

  /**
   * This method gets a <code>Font</code> with the specified size, style, strikeout and underline.
   * 
   * @param name
   * @param size
   * @param style
   * @param strikeout
   * @param underline
   * @return Font
   */
  @SuppressWarnings("unchecked")//$NON-NLS-1$
  public static Font getFont(final String name, final int size, final int style, final boolean strikeout, final boolean underline) {
    final StringBuffer fontName = new StringBuffer();
    fontName.append(name).append(Chars.PIPE);
    fontName.append(size).append(Chars.PIPE);
    fontName.append(style).append(Chars.PIPE);
    fontName.append(strikeout).append(Chars.PIPE);
    fontName.append(underline);
    final String fontN = fontName.toString();
    if (resources.containsKey(fontN)) {
      return (Font)resources.get(fontN);
    }
    final FontData fd = new FontData(name, size, style);
    if (strikeout || underline) {
      try {
        // *nix will now compile.
        final Class lfCls = Class.forName(ORG_ECLIPSE_SWT_INTERNAL_WIN32_LOGFONT);
        final Object lf = FontData.class.getField(DATA).get(fd);
        if ((lf != null) && (lfCls != null)) {
          if (strikeout) {
            lfCls.getField(LF_STRIKE_OUT).set(lf, new Byte((byte)1));
          }
          if (underline) {
            lfCls.getField(LF_UNDERLINE).set(lf, new Byte((byte)1));
          }
        }
      } catch (final Exception e) {
        log.error(Debug.getString(SWTResourceManager.class, 1), e);
      }
    }
    final Font font = new Font(Display.getDefault(), fd);
    resources.put(fontN, font);
    return font;
  }

  /**
   * This method gets an <code>Image</code> from the specified URL for a widget.
   * 
   * @param url
   * @param widget
   * @return Image
   */
  public static Image getImage(final String url, final Control widget) {
    final Image img = getImage(url);
    if ((img != null) && (widget != null)) {
      img.setBackground(widget.getBackground());
    }
    return img;
  }

  /**
   * This method gets an <code>Image</code> from the specified URL.
   * 
   * @param pUrl
   * @return Image
   */
  public static Image getImage(final String pUrl) {
    try {
      String url = pUrl.replace(Chars.BSLASH, Chars.SLASH);
      if ((url.length() > 0) && (url.charAt(0) == Chars.SLASH)) {
        url = url.substring(1);
      }
      if (resources.containsKey(url)) {
        return (Image)resources.get(url);
      }
      Image img = null;
      try {
        img = new Image(Display.getDefault(), instance.getClass().getClassLoader().getResourceAsStream(url));
      } catch (final Exception e) {
        img = new Image(Display.getDefault(), url);
      }
      resources.put(url, img);
      return img;
    } catch (final Exception e) {
      final List<String> replace = new ArrayList<String>();
      replace.add(pUrl);
      log.error(Debug.getString(SWTResourceManager.class, 2, replace), e);
      return null;
    }
  }

  /**
   * This method generates an image according to the percentage.
   * 
   * @param percentage
   * @param pForeground
   * @param pBackground
   * @param pFont
   * @param pText
   * @return Image
   */
  public static Image getImage(final double percentage, final Color pForeground, final Color pBackground, final Font pFont,
    final Color pText) {
    try {
      final PercentageImage pi = new PercentageImage(percentage, pForeground, pBackground, pFont, pText);
      resources.put(String.valueOf(percentage), pi.getImage());
      return pi.getImage();
    } catch (final Exception e) {
      final List<Double> replace = new ArrayList<Double>();
      replace.add(new Double(percentage));
      log.error(Debug.getString(SWTResourceManager.class, 3, replace), e);
      return null;
    }
  }

  /**
   * This method gets a <code>Color</code> with the specified red, green and blue components.
   * 
   * @param red
   * @param green
   * @param blue
   * @return Color
   */
  public static Color getColor(final int red, final int green, final int blue) {
    final String name = COLOR + red + Chars.COMMA + green + Chars.COMMA + blue;
    if (resources.containsKey(name)) {
      return (Color)resources.get(name);
    }
    final Color color = new Color(Display.getDefault(), red, green, blue);
    resources.put(name, color);
    return color;
  }

  /**
   * This method gets a <code>Cursor</code> with the specified type.
   * 
   * @param type
   * @return Cursor
   */
  public static Cursor getCursor(final int type) {
    final String name = CURSOR + type;
    if (resources.containsKey(name)) {
      return (Cursor)resources.get(name);
    }
    final Cursor cursor = new Cursor(Display.getDefault(), type);
    resources.put(name, cursor);
    return cursor;
  }
}