/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.concurrent.atomic.AtomicBoolean;

import net.za.grasser.duplicate.compare.Comparer;
import net.za.grasser.duplicate.file.DirectoryInfo;
import net.za.grasser.duplicate.file.DirectoryReader;
import net.za.grasser.duplicate.file.FingerprintFile;
import net.za.grasser.duplicate.persist.DatabaseFactory;
import net.za.grasser.duplicate.util.Chars;
import net.za.grasser.duplicate.util.Files;
import net.za.grasser.duplicate.util.Strings;
import net.za.grasser.duplicate.util.res.Debug;
import net.za.grasser.duplicate.util.res.Messages;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

/**
 * This class is the starting point of the command line checker.
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.26 $
 */
public class DuplicateChecker implements Observer, Runnable {
  /**
   * <code>VERSION</code> DuplicateChecker -
   */
  private static final String VERSION = "0.3.2"; //$NON-NLS-1$
  /**
   * <code>HELP</code> DuplicateChecker -
   */
  private static final String HELP = "--help"; //$NON-NLS-1$
  /**
   * <code>HELPQ</code> DuplicateChecker -
   */
  private static final String HELPQ = "-?"; //$NON-NLS-1$
  /**
   * <code>HELPH</code> DuplicateChecker -
   */
  private static final String HELPH = "-h"; //$NON-NLS-1$
  /** @modelguid {D4AF1AD2-EC0E-4C22-BC21-8C5DE9FFDBAC} */
  static Logger log;
  /**
   * <code>running</code> DuplicateChecker -
   */
  private AtomicBoolean running = new AtomicBoolean(false);
  /**
   * <code>thread</code> DuplicateChecker -
   */
  private Thread thread = null;
  static {
    Configure.load();
    PropertyConfigurator.configure(Files.LOG4J);
    log = Logger.getLogger(DuplicateChecker.class);
  }

  /**
   * Run as a command line application
   * 
   * @param args
   */
  private void runCommandLine(final String[] args) {
    String[] dirs = null;
    String wc = null;
    if (args.length <= 0) {
      dirs = new String[1];
      dirs[0] = Strings.DOT;
      wc = Strings.STAR;
    } else {
      if ((args[args.length - 1].indexOf(Chars.STAR) >= 0) || (args[args.length - 1].indexOf(Chars.QMARK) >= 0)
        || (args[args.length - 1].indexOf(Chars.OPN_SQ_BRACKET) >= 0)) {
        if (args.length == 1) {
          dirs = new String[1];
          dirs[0] = Strings.DOT;
        } else {
          dirs = new String[args.length - 1];
          for (int i = 0; i < dirs.length; i++) {
            dirs[i] = args[i];
          }
        }
        wc = args[args.length - 1];
      } else {
        dirs = new String[args.length];
        for (int i = 0; i < dirs.length; i++) {
          dirs[i] = args[i];
        }
        wc = Strings.STAR;
      }
    }
    if (wc.indexOf(Chars.OPN_SQ_BRACKET) == 0) {
      wc = wc.substring(1);
    }
    if (wc.endsWith(Strings.CLS_SQ_BRACKET)) {
      wc = wc.substring(0, wc.length() - 1);
    }
    final Comparer compare = Comparer.getInstance();
    compare.addObserver(this);
    compare.start();
    if (log.isDebugEnabled()) {
      final List<Object> replace = new ArrayList<Object>();
      replace.add(compare.getClass().getSimpleName());
      log.debug(Debug.getString(DuplicateChecker.class, 1, replace));
    }
    try {
      for (final String element : dirs) {
        if (log.isDebugEnabled()) {
          final List<Object> replace = new ArrayList<Object>();
          replace.add(wc);
          log.debug(Debug.getString(DuplicateChecker.class, 2, replace));
        }
        final DirectoryInfo df = new DirectoryInfo(element);
        if (log.isDebugEnabled()) {
          final List<Object> replace = new ArrayList<Object>();
          replace.add(df.getClass().getSimpleName());
          replace.add(element);
          log.debug(Debug.getString(DuplicateChecker.class, 3, replace));
        }
        final DirectoryReader dr = new DirectoryReader(wc, df);
        compare.addReader(dr);
        if (log.isDebugEnabled()) {
          final List<Object> replace = new ArrayList<Object>();
          replace.add(dr.getClass().getSimpleName());
          replace.add(wc);
          replace.add(element);
          log.debug(Debug.getString(DuplicateChecker.class, 4, replace));
        }
        dr.start();
      }
    } catch (final FileNotFoundException fnf) {
      log.error(Messages.getString(DuplicateChecker.class, 11), fnf);
      running.set(false);
    }
  }

  /**
   * This method prints usage info
   */
  private void printUsage() {
    final List<Object> replace = new ArrayList<Object>();
    replace.add(Strings.JAVA + DuplicateChecker.class.getName());
    log.warn(Messages.getString(DuplicateChecker.class, 5, replace));
    log.warn(Messages.getString(DuplicateChecker.class, 12));
    log.warn(Messages.getString(DuplicateChecker.class, 13));
  }

  /**
   * DuplicateChecker constructor
   * 
   * @param args
   */
  public DuplicateChecker(final String[] args) {
    if (log.isInfoEnabled()) {
      final List<Object> replace = new ArrayList<Object>();
      replace.add(DuplicateChecker.class.getSimpleName());
      replace.add(VERSION);
      log.info(Messages.getString(DuplicateChecker.class, 6, replace));
    }
    Configure.load();
    for (final String element : args) {
      if (element.equalsIgnoreCase(HELPQ) || element.startsWith(HELPH) || element.startsWith(HELP)) {
        printUsage();
        return;
      }
    }
    if (log.isInfoEnabled()) {
      log.info(Messages.getString(DuplicateChecker.class, 14));
    }
    running.set(true);
    thread = new Thread(this, DuplicateChecker.class.getSimpleName());
    runCommandLine(args);
    thread.start();
  }

  /**
   * @param args String[]
   */
  public static void main(final String[] args) {
    new DuplicateChecker(args);
  }

  /**
   * list duplicates
   */
  private void listDuplicates() {
    final StringBuffer sb = new StringBuffer(100);
    final Comparer compare = Comparer.getInstance();
    final List<Object> replace = new ArrayList<Object>();
    replace.add(new Integer(compare.getDuplicates().size()));
    log.warn(Messages.getString(DuplicateChecker.class, 7, replace));
    log.warn(Messages.getString(DuplicateChecker.class, 15));
    for (final FingerprintFile ff : compare.getDuplicates()) {
      sb.append(Strings.LS).append(ff.toString());
      // cut last LS
      sb.setLength(sb.length() - Strings.LS.length());
      log.warn(sb);
      // clear
      sb.setLength(0);
    }
  }

  /**
   * This method ...
   * 
   * @param simList
   * @param pfA
   * @param pfB
   */
  private void recursDuplicates(final SortedMap<Float, List<String>> simList, final FingerprintFile pfA, final FingerprintFile pfB) {
    if ((pfB != null) && ((pfA.getDuplicates() == null) || pfA.getDuplicates().isEmpty())) {
      final StringBuffer sb = new StringBuffer(100);
      sb.setLength(0);
      sb.append(pfB.getPath()).append(Chars.SPACE);
      sb.append(Chars.EQ).append(Chars.OPN_BRACKET).append(pfA.getSimilarity()).append(Chars.PERCENT).append(Chars.CLS_BRACKET).append(
        Chars.EQ);
      sb.append(Chars.SPACE).append(pfA.getPath());
      List<String> files = simList.get(new Float(pfA.getSimilarity()));
      if (files == null) {
        files = new ArrayList<String>();
        simList.put(new Float(pfA.getSimilarity()), files);
      }
      files.add(sb.toString());
      return;
    }
    for (final FingerprintFile ff : pfA.getDuplicates()) {
      recursDuplicates(simList, ff, pfA);
    }
  }

  /**
   * sort dulicates by similarity
   */
  private void listDuplicatesSorted() {
    final Comparer compare = Comparer.getInstance();
    final List<Object> replace = new ArrayList<Object>();
    replace.add(new Integer(compare.getDuplicates().size()));
    log.warn(Messages.getString(DuplicateChecker.class, 7, replace));
    log.warn(Messages.getString(DuplicateChecker.class, 16));
    final SortedMap<Float, List<String>> simList = new TreeMap<Float, List<String>>();
    final Iterator<FingerprintFile> it = compare.getDuplicates().iterator();
    while (it.hasNext()) {
      recursDuplicates(simList, it.next(), null);
    }
    final Iterator<Float> it5 = simList.keySet().iterator();
    while (it5.hasNext()) {
      final Iterator<String> it2 = simList.get(it5.next()).iterator();
      while (it2.hasNext()) {
        log.warn(it2.next());
      }
    }
  }

  /**
   * list all problem files
   */
  private void listProblems() {
    final Comparer compare = Comparer.getInstance();
    final StringBuffer sb = new StringBuffer(100);
    final List<Object> replace = new ArrayList<Object>();
    replace.add(new Integer(compare.getProblems().size()));
    log.warn(Messages.getString(DuplicateChecker.class, 8, replace));
    log.warn(Messages.getString(DuplicateChecker.class, 17));
    final Iterator<FingerprintFile> it = compare.getProblems().iterator();
    while (it.hasNext()) {
      final FingerprintFile f = it.next();
      sb.setLength(0);
      sb.append(f.getPath()).append(Chars.SPACE).append(Chars.OPN_SQ_BRACKET).append(f.getStatus().getDescription()).append(
        Chars.CLS_SQ_BRACKET);
      log.warn(sb);
    }
  }

  /**
   * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
   */
  public void update(final Observable pO, final Object pArg) {
    final List<Object> replace = new ArrayList<Object>();
    if (pArg instanceof FingerprintFile) {
      final FingerprintFile bf = (FingerprintFile)pArg;
      replace.add(Strings.LS + bf.toString());
      log.warn(Messages.getString(DuplicateChecker.class, 9, replace));
    } else if (pArg instanceof String) {
      final String str = (String)pArg;
      if ((pO instanceof Comparer) && str.equals(Strings.STOP)) {
        final Comparer compare = Comparer.getInstance();
        replace.add(new Integer(compare.getFiles().size()));
        log.warn(Messages.getString(DuplicateChecker.class, 10, replace));
        compare.stoppedReading();
        if (compare.getDuplicates().size() > 0) {
          listDuplicates();
          listDuplicatesSorted();
        }
        if (compare.getProblems().size() > 0) {
          listProblems();
        }
        DatabaseFactory.getDatabase().close();
        running.set(false);
      }
    }
  }

  /**
   * @see java.lang.Runnable#run()
   */
  public void run() {
    if (log.isDebugEnabled()) {
      log.debug(Debug.getString(DuplicateChecker.class, 18));
    }
    while (true) {
      if (running.get()) {
        synchronized (this) {
          try {
            wait(100L);
          } catch (final InterruptedException e) {
            break;
          }
        }
      } else {
        break;
      }
      try {
        if (System.in.available() > 0) {
          final int c = System.in.read();
          if ((c == Chars.q) || (c == Chars.Q)) {
            Comparer.getInstance().stop();
            running.set(false);
          }
        }
      } catch (final IOException e) {
        log.error(Debug.getString(DuplicateChecker.class, 19), e);
      }
    }
    if (log.isDebugEnabled()) {
      log.debug(Debug.getString(DuplicateChecker.class, 20));
    }
  }
}
