/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.regex.Pattern;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import net.za.grasser.duplicate.util.Chars;
import net.za.grasser.duplicate.util.RegExs;
import net.za.grasser.duplicate.util.Strings;
import net.za.grasser.duplicate.util.res.Messages;

/**
 * This class does the equivalent of jar -xf, but since jar.exe is not shipped with the JRE, I had to write my own!
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.16 $
 */
public class Jarx {
  /**
   * <code>DASH_R</code> Jarx -
   */
  private static final String DASH_R = "-r"; //$NON-NLS-1$
  /**
   * <code>LIB_DLL</code> Jarx -
   */
  private static final String LIB_DLL = "*.dll"; //$NON-NLS-1$
  /**
   * <code>LIB_JNILIB</code> Jarx -
   */
  private static final String LIB_JNILIB = "*.jnilib"; //$NON-NLS-1$
  /**
   * <code>LIB_SO</code> Jarx -
   */
  private static final String LIB_SO = "*.so"; //$NON-NLS-1$
  /**
   * <code>MESSAGE_1</code> Jarx - Default message 1, defined so that we get info while extracting.
   */
  public static final String MESSAGE_1 = "Usage: {0} path_to_jar [wildcard] [wildcard2] ..."; //$NON-NLS-1$
  /**
   * <code>MESSAGE_2</code> Jarx - Default message 2, defined so that we get info while extracting.
   */
  public static final String MESSAGE_2 = "Could not extract {0} because {1}"; //$NON-NLS-1$
  /**
   * <code>MESSAGE_3</code> Jarx - Default message 3, defined so that we get info while extracting.
   */
  public static final String MESSAGE_3 = "Extracting: {0}"; //$NON-NLS-1$
  /**
   * <code>MESSAGE_4</code> Jarx - Default message 4, defined so that we get info while extracting.
   */
  public static final String MESSAGE_4 = "RegEx: {0}"; //$NON-NLS-1$
  /**
   * <code>SLASH_R</code> Jarx -
   */
  private static final String SLASH_R = "/r"; //$NON-NLS-1$

  /**
   * Extracts all dll, so and jnilib files from pJar
   * 
   * @param pArgs
   * @throws Exception
   */
  @SuppressWarnings("unchecked")//$NON-NLS-1$
  private static void extract(final String[] pArgs) throws Exception {
    final File currentArchive = new File(pArgs[0]);
    final File outputDir = new File(RegExs.DOT_SLASH);
    final ZipFile zf = new ZipFile(currentArchive);
    final Enumeration<ZipEntry> entries = (Enumeration<ZipEntry>)zf.entries();
    final Pattern regex = makeRegex(pArgs);
    final List<String> replace = new ArrayList<String>();
    while (entries.hasMoreElements()) {
      final ZipEntry entry = entries.nextElement();
      final String pathname = entry.getName();
      if (regex.matcher(pathname).matches()) {
        final File outFile = new File(outputDir, pathname);
        if (!outFile.exists() || (entry.getTime() < 0L) || (entry.getTime() > outFile.lastModified())) {
          replace.clear();
          replace.add(pathname);
          System.out.println(Messages.getString(Jarx.class, 3, replace));
          if (!outFile.getParentFile().exists()) {
            outFile.getParentFile().mkdirs();
          }
          final InputStream in = zf.getInputStream(entry);
          final BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(outFile));
          final byte[] buf = new byte[1024];
          while (true) {
            final int nRead = in.read(buf, 0, buf.length);
            if (nRead <= 0) {
              break;
            }
            out.write(buf, 0, nRead);
          }
          in.close();
          out.close();
        }
      }
    }
    zf.close();
  }

  /**
   * @param args
   */
  public static void main(final String[] args) {
    final List<String> replace = new ArrayList<String>();
    if (args.length < 1) {
      replace.add(Strings.JAVA + Jarx.class.getSimpleName());
      System.out.println(Messages.getString(Jarx.class, 1, replace));
      System.exit(1);
    }
    try {
      Jarx.extract(args);
    } catch (final Exception e) {
      replace.clear();
      replace.add(args[0]);
      replace.add(e.getMessage());
      System.err.println(Messages.getString(Jarx.class, 2, replace));
      e.printStackTrace(System.err);
    }
  }

  /**
   * @param pArgs
   * @return a regular expression for the wildcards
   */
  private static Pattern makeRegex(final String[] pArgs) {
    final Set<String> s = new HashSet<String>();
    for (int i = 1; i < pArgs.length; i++) {
      s.add(pArgs[i]);
    }
    if (s.isEmpty()) {
      s.add(LIB_SO);
      s.add(LIB_DLL);
      s.add(LIB_JNILIB);
    }
    final StringBuffer reg = new StringBuffer(10);
    boolean more = false;
    for (final String t : s) {
      if (more) {
        reg.append(Chars.PIPE);
      }
      if (t.startsWith(DASH_R) || t.startsWith(SLASH_R)) {
        // already a regex
        reg.append(Chars.OPN_BRACKET);
        reg.append(t.substring(2));
        reg.append(Chars.CLS_BRACKET);
      } else {
        // a normal wildcard
        reg.append(Chars.OPN_BRACKET);
        reg.append(t.replaceAll(RegExs.BSLASH_DOT, RegExs.BSLASH_BSLASH_DOT).replaceAll(RegExs.BSLASH_STAR, RegExs.DOT_STAR).replaceAll(
          RegExs.BSLASH_QMARK, Strings.DOT));
        reg.append(Chars.DOLLAR).append(Chars.CLS_BRACKET);
      }
      more = true;
    }
    final List<StringBuffer> replace = new ArrayList<StringBuffer>();
    replace.add(reg);
    System.out.println(Messages.getString(Jarx.class, 4, replace));
    return Pattern.compile(reg.toString());
  }

  /**
   * private constructor for utility class
   */
  private Jarx() {
    super();
  }
}
