/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.compare;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.concurrent.atomic.AtomicBoolean;

import net.za.grasser.duplicate.file.FingerprintFile;
import net.za.grasser.duplicate.file.Status;
import net.za.grasser.duplicate.fingerprint.AbstractFingerprint;
import net.za.grasser.duplicate.persist.DatabaseFactory;
import net.za.grasser.duplicate.util.Chars;
import net.za.grasser.duplicate.util.Strings;
import net.za.grasser.duplicate.util.res.Debug;

import org.apache.log4j.Logger;

/**
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.31 $
 */
class CompareWorker extends Observable implements Runnable {
  /** @modelguid {2295ADEE-A695-4A57-ADD2-DD469FEA833F} */
  static Logger log = Logger.getLogger(CompareWorker.class);
  /**
   * <code>running</code> Comparer -
   */
  private AtomicBoolean running = new AtomicBoolean(false);
  /**
   * <code>number</code> Comparer -
   */
  private int number;
  /**
   * <code>parent</code> CompareWorker - refence to the Comparer singleton (othrewise we will have to call getInstance)
   */
  private Comparer parent;

  /**
   * Constructor
   * 
   * @param pParent
   * @param pNum
   */
  public CompareWorker(final Comparer pParent, final int pNum) {
    super();
    number = pNum;
    parent = pParent;
    start();
  }

  /**
   * Start the thread with minimum thread priority otherwise it slows them machine dramitically.
   */
  public void start() {
    if (!running.get()) {
      running.set(true);
      final Thread t = new Thread(this, CompareWorker.class.getSimpleName() + Chars.COLON + number);
      t.setDaemon(true);
      t.setPriority(Thread.MIN_PRIORITY);
      t.start();
    }
  }

  /**
   * This method generates the fingerprints
   * 
   * @param ff
   */
  private void process(final FingerprintFile ff) {
    boolean process = !DatabaseFactory.getDatabase().get(ff);
    if (ff.getStatus() == Status.FILE_UNKNOWN) {
      ff.setStatus(Status.FILE_OK); // fix old unknown files
      process = true;
    }
    if (process) {
      for (final String key : ff.getFingerprints().keySet()) {
        final AbstractFingerprint af = ff.getFingerprints().get(key);
        af.getFingerprint();
      }
      DatabaseFactory.getDatabase().put(ff);
    }
    ff.logFileStatus();
  }

  /**
   * This method processes the files.
   */
  private void process() {
    try {
      final FingerprintFile ff = parent.removeFirst();
      if ((ff == null) || (ff == Comparer.POISON)) {
        running.set(false);
        return;
      }
      process(ff);
      parent.checkIn(ff);
    } catch (final Error e) {
      log.error(Debug.getString(CompareWorker.class, 1), e);
      running.set(false);
    } catch (final Exception e) {
      log.warn(Debug.getString(CompareWorker.class, 5), e);
    }
  }

  /**
   * @see java.lang.Runnable#run()
   */
  public void run() {
    try {
      if (log.isDebugEnabled()) {
        final List<Integer> replace = new ArrayList<Integer>();
        replace.add(new Integer(number));
        log.debug(Debug.getString(CompareWorker.class, 2, replace));
      }
      while (running.get()) {
        process();
      }
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(CompareWorker.class, 3));
      }
    } catch (Throwable t) {
      log.error(Debug.getString(CompareWorker.class, 6), t);
    } finally {
      running.set(false);
      setChanged();
      super.notifyObservers(Strings.STOP);
      // try to poison the controller (it will only stop when all workers have sent a poison)
      parent.checkIn(Comparer.POISON);
      if (log.isDebugEnabled()) {
        final List<Integer> replace = new ArrayList<Integer>();
        replace.add(new Integer(number));
        log.debug(Debug.getString(CompareWorker.class, 4, replace));
      }
    }
  }

  /**
   * @return boolean - Returns the running.
   */
  public boolean isRunning() {
    return running.get();
  }

  /**
   * Stops all threads.
   */
  public void stop() {
    running.set(false);
  }
}
