/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.file;

import net.za.grasser.duplicate.util.RegExs;

/**
 * This class keeps a count of files that have the same extenstion.
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.5 $
 */
public class ExtensionCount implements Comparable<Object> {
  /**
   * <code>extension</code> ExtensionCount -
   */
  private String extension = null;
  /**
   * <code>count</code> ExtensionCount -
   */
  private int count = 1;

  /**
   * @param pExt
   */
  public ExtensionCount(final String pExt) {
    super();
    if (!pExt.startsWith(RegExs.STAR_DOT)) {
      extension = RegExs.STAR_DOT + pExt;
    } else {
      extension = pExt;
    }
  }

  /**
   * @see java.lang.Comparable#compareTo(java.lang.Object)
   */
  public int compareTo(final Object arg0) {
    if (arg0 instanceof ExtensionCount) {
      return extension.compareTo(((ExtensionCount)arg0).extension);
    }
    return -1;
  }

  /**
   * @return int - Returns the count.
   */
  public int getCount() {
    return count;
  }

  /**
   * increments the count by 1.
   */
  public void incCount() {
    count++;
  }

  /**
   * @return String - Returns the extension.
   */
  public String getExtension() {
    return extension;
  }

  /**
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString() {
    return extension;
  }

  /**
   * @see java.lang.Object#equals(java.lang.Object)
   */
  @Override
  public boolean equals(final Object pObj) {
    if (pObj instanceof ExtensionCount) {
      return extension.equals(((ExtensionCount)pObj).extension);
    }
    return false;
  }
}
