/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.file;

import java.io.Serializable;

import net.za.grasser.duplicate.util.Chars;

/**
 * This enum defines the statuses that a FingerprintFile can have.
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 */
public enum Status implements Serializable {
  /**
   * <code>FILE_MISSING</code> Status -
   */
  FILE_MISSING(7, "FILE_MISSING"), //$NON-NLS-1$
  /**
   * <code>FILE_EMPTY</code> Status -
   */
  FILE_EMPTY(6, "FILE_EMPTY"), //$NON-NLS-1$
  /**
   * <code>FILE_CORRUPT</code> Status -
   */
  FILE_CORRUPT(5, "FILE_CORRUPT"), //$NON-NLS-1$
  /**
   * <code>FILE_INVALID</code> Status -
   */
  FILE_INVALID(4, "FILE_INVALID"), //$NON-NLS-1$
  /**
   * <code>FILE_NOT_READABLE</code> Status -
   */
  FILE_NOT_READABLE(3, "FILE_NOT_READABLE"), //$NON-NLS-1$
  /**
   * <code>FILE_SIGNATURE_MISMATCH</code> Status -
   */
  FILE_SIGNATURE_MISMATCH(2, "FILE_SIGNATURE_MISMATCH"), //$NON-NLS-1$
  /**
   * <code>FILE_TOO_SHORT</code> Status -
   */
  FILE_TOO_SHORT(1, "FILE_TOO_SHORT"), //$NON-NLS-1$
  /**
   * <code>FILE_OK</code> Status -
   */
  FILE_OK(0, "FILE_OK"), //$NON-NLS-1$
  /**
   * <code>FILE_UNKNOWN</code> Status -
   */
  FILE_UNKNOWN(-1, "FILE_UNKNOWN"); //$NON-NLS-1$
  /**
   * <code>value</code> Status - internal value (also used in the db)
   */
  private final int value;
  /**
   * <code>description</code> Status -
   */
  private final String description;

  /**
   * Sole constructor.
   * 
   * @param pValue
   * @param pDesc
   */
  private Status(final int pValue, final String pDesc) {
    value = pValue;
    description = pDesc;
  }

  /**
   * This method ...
   * 
   * @param pVal
   * @return Status
   */
  public static Status find(final int pVal) {
    for (final Status lStat : values()) {
      if (lStat.value == pVal) {
        return lStat;
      }
    }
    return FILE_UNKNOWN;
  }

  /**
   * @return the value
   */
  public int getValue() {
    return value;
  }

  /**
   * @return the description
   */
  public String getDescription() {
    return description;
  }

  /**
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString() {
    return description + Chars.SPACE + Chars.OPN_BRACKET + value + Chars.CLS_BRACKET;
  }
}