/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.file;

import java.io.File;
import java.io.FilenameFilter;
import java.util.regex.Pattern;

import net.za.grasser.duplicate.Configure;
import net.za.grasser.duplicate.util.Chars;
import net.za.grasser.duplicate.util.RegExs;
import net.za.grasser.duplicate.util.Strings;

/**
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.14 $
 */
public class WildcardFilter implements FilenameFilter {
  /**
   * <code>IGNORE_DOT_FILES</code> DirectoryReader -
   */
  private static final String IGNORE_DOT_FILES = "ignore .files";//$NON-NLS-1$
  /**
   * <code>CASE_SENSITIVE</code> DirectoryReader -
   */
  private static final String CASE_SENSITIVE = "case sensitive"; //$NON-NLS-1$
  /**
   * <code>MATCH_DIRS</code> DirectoryReader -
   */
  private static final String MATCH_DIRS = "match dirs"; //$NON-NLS-1$
  /**
   * <code>dotFiles</code> WildcardFilter - ignore .files
   */
  private static boolean ignoringDotFiles = true;
  /**
   * <code>matchdirs</code> DirectoryReader - match directories to wildcardFilter (Unix style)
   */
  private static boolean matchDirs = false;
  /**
   * <code>caseSensitive</code> DirectoryReader - case sensitivley match files
   */
  private static boolean caseSensitive = false;
  static {
    Configure.load();
    ignoringDotFiles = Configure.getBooleanProperty(IGNORE_DOT_FILES, ignoringDotFiles, WildcardFilter.class.getName());
    matchDirs = Configure.getBooleanProperty(MATCH_DIRS, matchDirs, DirectoryReader.class.getName());
    caseSensitive = Configure.getBooleanProperty(CASE_SENSITIVE, caseSensitive, DirectoryReader.class.getName());
  }

  /**
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString() {
    return new String(regex.pattern());
  }

  /** @modelguid {E253A5F7-ADD8-43C2-86A1-1645ADD2DEF4} */
  Pattern regex;

  /**
   * @param str String - wildcard expression
   */
  public WildcardFilter(final String str) {
    if (caseSensitive) {
      regex = Pattern.compile(createRegEx(str));
    } else {
      regex = Pattern.compile(createRegEx(str), Pattern.CASE_INSENSITIVE);
    }
  }

  /**
   * This method turns a wildcard string into a regex string. Only ? and * are handled.<BR>
   * TODO expand to handle globbing
   * 
   * @param str
   * @return regular expression string created from a wildcard string.
   */
  public static String createRegEx(final String str) {
    final StringBuffer regex = new StringBuffer();
    regex.append(Chars.CFLEX);
    for (int i = 0; i < str.length(); i++) {
      switch (str.charAt(i)) {
        case Chars.STAR:
          regex.append(RegExs.DOT_STAR);
          break;
        case Chars.QMARK:
          regex.append(Strings.DOT);
          break;
        case Chars.DOT:
          regex.append(RegExs.BSLASH_DOT);
          break;
        default:
          regex.append(str.charAt(i));
          break;
      }
    }
    regex.append(Chars.DOLLAR);
    return regex.toString();
  }

  /**
   * @param pRegex Pattern - use a regex directly.
   */
  public WildcardFilter(final Pattern pRegex) {
    regex = pRegex;
  }

  /**
   * @see java.io.FilenameFilter#accept(java.io.File, java.lang.String)
   */
  public boolean accept(final File dir, final String name) {
    if (ignoringDotFiles && (name.charAt(0) == Chars.DOT)) {
      return false;
    }
    if (!matchDirs && (new File(dir.getPath() + File.separatorChar + name)).isDirectory()) {
      return true;
    }
    return regex.matcher(name).matches();
  }

  /**
   * @return boolean - Returns the ignoringDotFiles.
   */
  public static boolean isIgnoringDotFiles() {
    return ignoringDotFiles;
  }

  /**
   * @return boolean - Returns the caseSensitive.
   */
  public static boolean isCaseSensitive() {
    return caseSensitive;
  }

  /**
   * @return boolean - Returns the matchdirs.
   */
  public static boolean isMatchdirs() {
    return matchDirs;
  }
}
