/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.fingerprint.configure;

import java.awt.image.AffineTransformOp;

import net.za.grasser.duplicate.Configure;
import net.za.grasser.duplicate.util.res.Debug;
import net.za.grasser.duplicate.util.res.Messages;

import org.apache.log4j.Logger;

/**
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.15 $
 */
public class ColorFingerprintConfig extends FingerprintConfig {
  /**
   * <code>BICUBIC</code> ColorFingerprintConfig -
   */
  private static final String BICUBIC = "BICUBIC"; //$NON-NLS-1$ 
  /**
   * <code>BILINEAR</code> ColorFingerprintConfig -
   */
  private static final String BILINEAR = "BILINEAR"; //$NON-NLS-1$ 
  /**
   * <code>NEAREST_NEIGHBOR</code> ColorFingerprintConfig -
   */
  private static final String NEAREST_NEIGHBOR = "NEAREST NEIGHBOR"; //$NON-NLS-1$ 
  /**
   * <code>SCALING_ALGORITHM</code> ColorFingerprintConfig -
   */
  private static final String SCALING_ALGORITHM = "scaling algorithm"; //$NON-NLS-1$ 
  /**
   * <code>KEEP_ASPECT</code> ColorFingerprintConfig -
   */
  private static final String KEEP_ASPECT = "keep aspect"; //$NON-NLS-1$ 
  /**
   * <code>SUBDIVISION_LEVEL</code> ColorFingerprintConfig -
   */
  private static final String SUBDIVISION_LEVEL = "subdivision level"; //$NON-NLS-1$ 

  /**
   * This class ...
   * 
   * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
   * @version $Revision: 1.15 $
   */
  static class Factory extends ConfigFactory {
    /**
     * @see net.za.grasser.duplicate.fingerprint.configure.ConfigFactory#create()
     */
    @Override
    protected FingerprintConfig create() {
      return ColorFingerprintConfig.getInstance();
    }
  }

  static {
    ConfigFactory.addFactory(ColorFingerprintConfig.class.getSimpleName(), new Factory());
  }
  /**
   * <code>serialVersionUID</code> ColorFingerprintConfig.java -
   */
  private static final long serialVersionUID = 4351025227180155410L;
  /**
   * <code>log</code> ColorFingerprintConfig -
   */
  private static final Logger log = Logger.getLogger(ColorFingerprintConfig.class);
  /**
   * <code>fc</code> ColorFingerprintConfig -
   */
  private static FingerprintConfig fc = null;

  /**
   * @return FingerprintConfig
   */
  public final static FingerprintConfig getInstance() {
    if (fc == null) {
      fc = new ColorFingerprintConfig();
    }
    return fc;
  }

  /**
   * <code>subdivide</code> ColorFingerprintConfig - 1=>1 tile, 2=>4 tiles, 3=>16 tiles
   */
  private int subdivide = 3;
  /**
   * <code>keepaspect</code> ColorFingerprintConfig - keep the aspect ratio makes comparrison faster by 2
   */
  private boolean keepaspect = true;
  /**
   * <code>scaling_alg</code> ColorFingerprint - scaling algorithm
   */
  private int scalingAlgorithm = AffineTransformOp.TYPE_BILINEAR;
  /**
   * <code>scalings</code> ColorFingerprintConfig -
   */
  private final String[] scalings = {
    NEAREST_NEIGHBOR, BILINEAR, BICUBIC
  };

  /**
   * 
   */
  private ColorFingerprintConfig() {
    super();
    setSubdivide(Configure.getIntProperty(SUBDIVISION_LEVEL, subdivide, ColorFingerprintConfig.class.getName()));
    setKeepaspect(Configure.getBooleanProperty(KEEP_ASPECT, keepaspect, ColorFingerprintConfig.class.getName()));
    setScalingAlgorithm(Configure.getProperty(SCALING_ALGORITHM, NEAREST_NEIGHBOR, ColorFingerprintConfig.class.getName()));
  }

  /**
   * @see net.za.grasser.duplicate.fingerprint.configure.FingerprintConfig#getFingerprintName()
   */
  @Override
  public String getFingerprintName() {
    return Messages.getString(ColorFingerprintConfig.class, 1);
  }

  /**
   * @return int - Returns the scalingAlgorithm.
   */
  public int getScalingAlgorithm() {
    return scalingAlgorithm;
  }

  /**
   * @return String[] - Returns the list of scalingAlgorithms.
   */
  public String[] getScalingAlgorithms() {
    return scalings;
  }

  /**
   * @return String
   */
  public final int getSubdivide() {
    return subdivide;
  }

  /**
   * @return boolean - Returns the keepaspect.
   */
  public boolean isKeepaspect() {
    return keepaspect;
  }

  /**
   * Sets the keepaspect to the value of pKeepaspect. F
   * 
   * @param pKeepaspect boolean - The new value.
   */
  public void setKeepaspect(final boolean pKeepaspect) {
    keepaspect = pKeepaspect;
  }

  /**
   * Sets the scalingAlgorithm to the value of pScalingAlgorithm.
   * 
   * @param pScalingAlgorithm int - The new value.
   */
  public void setScalingAlgorithm(final String pScalingAlgorithm) {
    final String alg = pScalingAlgorithm.toUpperCase();
    try {
      if (alg.startsWith(NEAREST_NEIGHBOR.substring(0, 3))) {
        scalingAlgorithm = AffineTransformOp.TYPE_NEAREST_NEIGHBOR;
      } else if (alg.startsWith(BILINEAR.substring(0, 3))) {
        scalingAlgorithm = AffineTransformOp.TYPE_BILINEAR;
      } else if (alg.startsWith(BICUBIC.substring(0, 3))) {
        scalingAlgorithm = AffineTransformOp.TYPE_BICUBIC;
      } else {
        scalingAlgorithm = AffineTransformOp.TYPE_NEAREST_NEIGHBOR;
      }
    } catch (final RuntimeException re) {
      log.warn(Debug.getString(ColorFingerprintConfig.class, 2), re);
      scalingAlgorithm = AffineTransformOp.TYPE_NEAREST_NEIGHBOR;
    }
  }

  /**
   * This method ...
   * 
   * @param pScalingAlgorithm
   */
  public void setScalingAlgorithm(final int pScalingAlgorithm) {
    scalingAlgorithm = pScalingAlgorithm;
  }

  /**
   * @param pSub
   */
  public void setSubdivide(final int pSub) {
    subdivide = pSub;
  }
}
