/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.fingerprint.configure;

import net.za.grasser.duplicate.Configure;
import net.za.grasser.duplicate.util.fft.FFT;
import net.za.grasser.duplicate.util.res.Messages;

/**
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.12 $
 */
public class FdmfFingerprintConfig extends FingerprintConfig {
  /**
   * <code>ALPHA</code> FdmfFingerprintConfig -
   */
  private static final String ALPHA = "alpha"; //$NON-NLS-1$ 
  /**
   * <code>WINDOW</code> FdmfFingerprintConfig -
   */
  private static final String WINDOW = "window"; //$NON-NLS-1$ 
  /**
   * <code>KAISER</code> FdmfFingerprintConfig -
   */
  private static final String KAISER = "KAISER"; //$NON-NLS-1$ 
  /**
   * <code>WELCH</code> FdmfFingerprintConfig -
   */
  private static final String WELCH = "WELCH"; //$NON-NLS-1$ 
  /**
   * <code>BARTLETT</code> FdmfFingerprintConfig -
   */
  private static final String BARTLETT = "BARTLETT"; //$NON-NLS-1$ 
  /**
   * <code>TRIANGULAR</code> FdmfFingerprintConfig -
   */
  private static final String TRIANGULAR = "TRIANGULAR"; //$NON-NLS-1$ 
  /**
   * <code>RAISED_COSINE</code> FdmfFingerprintConfig -
   */
  private static final String RAISED_COSINE = "RAISED COSINE"; //$NON-NLS-1$ 
  /**
   * <code>HANNING</code> FdmfFingerprintConfig -
   */
  private static final String HANNING = "HANNING"; //$NON-NLS-1$ 
  /**
   * <code>HAMMING</code> FdmfFingerprintConfig -
   */
  private static final String HAMMING = "HAMMING"; //$NON-NLS-1$ 
  /**
   * <code>RECTANGULAR</code> FdmfFingerprintConfig -
   */
  private static final String RECTANGULAR = "RECTANGULAR"; //$NON-NLS-1$ 
  /**
   * <code>serialVersionUID</code> FdmfFingerprintConfig.java -
   */
  private static final long serialVersionUID = -1603772768877742066L;
  /**
   * <code>fc</code> FdmfFingerprintConfig -
   */
  private static FingerprintConfig fc = null;
  /**
   * <code>scaling_alg</code> FdmfFingerprintConfig - scaling algorithm
   */
  private int windowType = FdmfFingerprintConfig.WINDOW_RECTANGULAR;
  /**
   * <code>alpha</code> FdmfFingerprintConfig - modifier for Raised Cosine and Kaiser windows
   */
  private double alpha = 5.0;
  /**
   * <code>window</code> FdmfFingerprintConfig - actual precalculated window values according to windowType
   */
  private double[] window;
  /**
   * <code>windowTypes</code> FdmfFingerprintConfig -
   */
  private final String[] windowTypes = {
    RECTANGULAR, HAMMING, HANNING, RAISED_COSINE, BARTLETT, WELCH, KAISER
  };
  /**
   * <code>CHUNK_SAMPLES</code> FdmfFingerprint - approx. 375 milliseconds
   */
  public static final int CHUNK_SAMPLES = 16384;
  /**
   * <code>WINDOW_KAISER</code> FdmfFingerprint -
   */
  public static final int WINDOW_KAISER = 6;
  /**
   * <code>WINDOW_BARTLETT</code> FdmfFingerprint -
   */
  public static final int WINDOW_BARTLETT = 5;
  /**
   * <code>WINDOW_WELCH</code> FdmfFingerprint -
   */
  public static final int WINDOW_WELCH = 4;
  /**
   * <code>WINDOW_RAISED_COSINE</code> FdmfFingerprint -
   */
  public static final int WINDOW_RAISED_COSINE = 3;
  /**
   * <code>WINDOW_HAMMING</code> FdmfFingerprint -
   */
  public static final int WINDOW_HAMMING = 2;
  /**
   * <code>WINDOW_HANNING</code> FdmfFingerprint -
   */
  public static final int WINDOW_HANNING = 1;
  /**
   * <code>WINDOW_RECTANGULAR</code> FdmfFingerprint -
   */
  public static final int WINDOW_RECTANGULAR = 0;

  /**
   * This class ...
   * 
   * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
   * @version $Revision: 1.12 $
   */
  static class Factory extends ConfigFactory {
    /**
     * @see net.za.grasser.duplicate.fingerprint.configure.ConfigFactory#create()
     */
    @Override
    protected FingerprintConfig create() {
      return FdmfFingerprintConfig.getInstance();
    }
  }

  static {
    ConfigFactory.addFactory(FdmfFingerprintConfig.class.getSimpleName(), new Factory());
  }

  /**
   * 
   */
  private FdmfFingerprintConfig() {
    super();
    setWindowType(Configure.getProperty(WINDOW, RECTANGULAR, FdmfFingerprintConfig.class.getName()));
  }

  /**
   * @return FingerprintConfig
   */
  public final static FingerprintConfig getInstance() {
    if (fc == null) {
      fc = new FdmfFingerprintConfig();
    }
    return fc;
  }

  /**
   * @return int - Returns the current window type.
   */
  public int getWindowType() {
    return windowType;
  }

  /**
   * @return int - Returns the list of window types.
   */
  public String[] getWindowTypes() {
    return windowTypes;
  }

  /**
   * Sets the windowType to the translated value of pWindowType.
   * 
   * @param pWindowType int - The new value.
   */
  public void setWindowType(final String pWindowType) {
    final String win = pWindowType.toUpperCase();
    if (win.startsWith(HAMMING.substring(0, 3))) {
      windowType = FdmfFingerprintConfig.WINDOW_HAMMING;
    } else if (win.startsWith(HANNING.substring(0, 3))) {
      windowType = FdmfFingerprintConfig.WINDOW_HANNING;
    } else if (win.startsWith(RAISED_COSINE.substring(0, 3))) {
      windowType = FdmfFingerprintConfig.WINDOW_RAISED_COSINE;
      setAlpha(Configure.getDoubleProperty(ALPHA, 0.5, FdmfFingerprintConfig.class.getName()));
    } else if (win.startsWith(BARTLETT.substring(0, 3)) || win.startsWith(TRIANGULAR.substring(0, 3))) {
      windowType = FdmfFingerprintConfig.WINDOW_BARTLETT;
    } else if (win.startsWith(WELCH.substring(0, 3))) {
      windowType = FdmfFingerprintConfig.WINDOW_WELCH;
    } else if (win.startsWith(KAISER.substring(0, 3))) {
      windowType = FdmfFingerprintConfig.WINDOW_KAISER;
      setAlpha(Configure.getDoubleProperty(ALPHA, alpha, FdmfFingerprintConfig.class.getName()));
    } else {
      windowType = FdmfFingerprintConfig.WINDOW_RECTANGULAR;
    }
    window = makeWindow(FdmfFingerprintConfig.CHUNK_SAMPLES);
  }

  /**
   * Sets the windowType to the value of pType
   * 
   * @param pType
   */
  public void setWindowType(final int pType) {
    windowType = pType;
  }

  /**
   * @return double
   */
  public double getAlpha() {
    return alpha;
  }

  /**
   * @param pAlpha
   */
  public void setAlpha(final double pAlpha) {
    alpha = pAlpha;
  }

  /**
   * @return double[]
   */
  public double[] getWindow() {
    return window;
  }

  /**
   * Creates a windowType table acording tot he windowType definition set up.
   * 
   * @param length
   * @return the windowType table.
   */
  public double[] makeWindow(final int length) {
    double[] d = null;
    switch (windowType) {
      default:
      case FdmfFingerprintConfig.WINDOW_RECTANGULAR:
        break;
      case FdmfFingerprintConfig.WINDOW_HAMMING:
        d = FFT.setupHammingWindow(length);
        break;
      case FdmfFingerprintConfig.WINDOW_HANNING:
        d = FFT.setupHanningWindow(length);
        break;
      case FdmfFingerprintConfig.WINDOW_RAISED_COSINE:
        d = FFT.setupRaisedCosineWindow(length, alpha);
        break;
      case FdmfFingerprintConfig.WINDOW_BARTLETT:
        d = FFT.setupBartlettWindow(length);
        break;
      case FdmfFingerprintConfig.WINDOW_WELCH:
        d = FFT.setupWelchWindow(length);
        break;
      case FdmfFingerprintConfig.WINDOW_KAISER:
        d = FFT.setupKaiserWindow(length, alpha);
        break;
    }
    return d;
  }

  /**
   * @see net.za.grasser.duplicate.fingerprint.configure.FingerprintConfig#getFingerprintName()
   */
  @Override
  public String getFingerprintName() {
    return Messages.getString(FdmfFingerprintConfig.class, 1);
  }
}
