/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.gui;

import java.io.File;
import java.util.Arrays;

import net.za.grasser.duplicate.file.FileComp;
import net.za.grasser.duplicate.file.WildcardFilter;
import net.za.grasser.duplicate.util.Files;
import net.za.grasser.duplicate.util.RegExs;
import net.za.grasser.duplicate.util.Strings;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

/**
 * This class ...
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.13 $
 */
public class FileTreeContentProvider implements ITreeContentProvider {
  /**
   * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
   */
  public Object[] getChildren(final Object element) {
    return getChildren(element, true);
  }

  /**
   * This method will return the children sorted or unsorted.
   * 
   * @param element
   * @param bSort
   * @return Object[]
   */
  public Object[] getChildren(final Object element, final boolean bSort) {
    if ((element == null) || ((File)element).getName().equals(Strings.STAR)) {
      return File.listRoots();
    }
    if (!((File)element).getPath().startsWith(Files.A_COLON) && !((File)element).getPath().startsWith(Files.B_COLON)) {
      final File[] kids = ((File)element).listFiles(new WildcardFilter(RegExs.STAR_DOT_STAR));
      if (kids == null) {
        return new Object[0];
      }
      if (bSort) {
        try {
          Arrays.sort(kids, new FileComp());
        } catch (final Exception e) {
          // ignore - return unsorted
        }
      }
      return kids;
    }
    return new Object[0];
  }

  /**
   * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
   */
  public Object getParent(final Object element) {
    return ((File)element).getParent();
  }

  /**
   * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
   */
  public boolean hasChildren(final Object element) {
    if (element != null) {
      return ((File)element).isDirectory();
    }
    return false;
  }

  /**
   * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
   */
  public Object[] getElements(final Object element) {
    return getChildren(element);
  }

  /**
   * @see org.eclipse.jface.viewers.IContentProvider#dispose()
   */
  public void dispose() {
    // Auto-generated method stub
  }

  /**
   * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
   */
  public void inputChanged(final Viewer element, final Object pArg1, final Object pArg2) {
    // Auto-generated method stub
  }
}
