/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.gui;

import java.util.Iterator;
import java.util.List;

import net.za.grasser.duplicate.file.ExtensionCount;
import net.za.grasser.duplicate.fingerprint.AbstractFingerprint;
import net.za.grasser.duplicate.fingerprint.ColorFingerprint;
import net.za.grasser.duplicate.fingerprint.FdmfFingerprint;
import net.za.grasser.duplicate.fingerprint.FingerprintFactory;
import net.za.grasser.duplicate.fingerprint.SifFingerprint;
import net.za.grasser.duplicate.fingerprint.SubstringFingerprint;
import net.za.grasser.duplicate.util.Files;
import net.za.grasser.duplicate.util.Strings;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.swt.graphics.Image;

import com.cloudgarden.resource.SWTResourceManager;

/**
 * This class ...
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.12 $
 */
public class TypeTableLabelProvider implements ITableLabelProvider {
  /**
   * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
   */
  public String getColumnText(final Object element, final int column_index) {
    if (column_index == 0) {
      return element.toString();
    }
    if (column_index == 1) {
      if (element instanceof ExtensionCount) {
        return String.valueOf(((ExtensionCount)element).getCount());
      }
    }
    return Strings.EMPTY;
  }

  /**
   * @see org.eclipse.jface.viewers.IBaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
   */
  public void addListener(final ILabelProviderListener ilabelproviderlistener) {
    // do nothing yet
  }

  /**
   * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
   */
  public void dispose() {
    // do nothing yet
  }

  /**
   * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object, java.lang.String)
   */
  public boolean isLabelProperty(final Object obj, final String s) {
    return false;
  }

  /**
   * @see org.eclipse.jface.viewers.IBaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
   */
  public void removeListener(final ILabelProviderListener ilabelproviderlistener) {
    // do nothing yet
  }

  /**
   * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
   */
  public Image getColumnImage(final Object element, final int column_index) {
    if (column_index != 0) {
      return null;
    }
    final List<Class< ? extends AbstractFingerprint>> s = FingerprintFactory.getFingerprintTypes(element.toString());
    final Iterator<Class< ? extends AbstractFingerprint>> it = s.iterator();
    while (it.hasNext()) {
      final Class< ? > cls = it.next();
      if (cls.getName().indexOf(ColorFingerprint.class.getSimpleName()) >= 0) {
        return SWTResourceManager.getImage(Files.ICONS_IMAGE_GIF);
      }
      if (cls.getName().indexOf(FdmfFingerprint.class.getSimpleName()) >= 0) {
        return SWTResourceManager.getImage(Files.ICONS_AUDIO_GIF);
      }
      if ((cls.getName().indexOf(SifFingerprint.class.getSimpleName()) >= 0)
        || (cls.getName().indexOf(SubstringFingerprint.class.getSimpleName()) >= 0)) {
        return SWTResourceManager.getImage(Files.ICONS_TEXT_GIF);
      }
    }
    return SWTResourceManager.getImage(Files.ICONS_BINARY_GIF);
  }
}
