/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.gui.config;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.beans.SimpleBeanInfo;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.za.grasser.duplicate.fingerprint.AbstractFingerprint;
import net.za.grasser.duplicate.fingerprint.configure.ConfigFactory;
import net.za.grasser.duplicate.fingerprint.configure.FingerprintConfig;
import net.za.grasser.duplicate.util.Chars;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.Viewer;

/**
 * This class ...
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.8 $
 */
public class ConfigContentProvider implements IStructuredContentProvider {
  /**
   * <code>ENUMERATION_VALUES</code> ConfigContentProvider -
   */
  private static final String ENUMERATION_VALUES = "enumerationValues"; //$NON-NLS-1$
  /**
   * <code>GET_INSTANCE</code> ConfigContentProvider -
   */
  private static final String GET_INSTANCE = "getInstance"; //$NON-NLS-1$
  /**
   * <code>SET</code> ConfigContentProvider -
   */
  private static final String SET = "set"; //$NON-NLS-1$
  /**
   * <code>HAS</code> ConfigContentProvider -
   */
  private static final String HAS = "has"; //$NON-NLS-1$
  /**
   * <code>IS</code> ConfigContentProvider -
   */
  private static final String IS = "is"; //$NON-NLS-1$
  /**
   * <code>GET</code> ConfigContentProvider -
   */
  private static final String GET = "get"; //$NON-NLS-1$
  /**
   * <code>BEAN_INFO</code> ConfigContentProvider -
   */
  private static final String BEAN_INFO = "BeanInfo"; //$NON-NLS-1$

  /**
   * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
   */
  @SuppressWarnings("unchecked")//$NON-NLS-1$
  public Object[] getElements(final Object element) {
    if (element == null) {
      return new Object[0];
    }
    String config = null;
    if (element instanceof Class) {
      config = ConfigFactory.getConfig((Class< ? extends AbstractFingerprint>)element).getClass().getName();
    }
    if (element instanceof FingerprintConfig) {
      config = ((FingerprintConfig)element).getClass().getName();
    }
    if (config == null) {
      return new Object[0];
    }
    final String beanInfo = config + BEAN_INFO;
    try {
      final Class< ? extends FingerprintConfig> fc = (Class< ? extends FingerprintConfig>)Class.forName(config);
      final Class< ? extends SimpleBeanInfo> c = (Class< ? extends SimpleBeanInfo>)Class.forName(beanInfo);
      final SimpleBeanInfo info = c.newInstance();
      final BeanInfo[] more = info.getAdditionalBeanInfo();
      final List<ConfigProperty> full = new ArrayList<ConfigProperty>();
      for (final PropertyDescriptor lDescriptor : info.getPropertyDescriptors()) {
        full.add(new ConfigProperty(lDescriptor, fc));
      }
      for (final BeanInfo bi : more) {
        for (final PropertyDescriptor lDescriptor : bi.getPropertyDescriptors()) {
          full.add(new ConfigProperty(lDescriptor, fc));
        }
      }
      Collections.sort(full, new ConfigPropertyComparator());
      return full.toArray();
    } catch (final ClassNotFoundException e) {
      // this is the case where the BeanInfo class does not exist
      try {
        final Class< ? extends FingerprintConfig> fc = (Class< ? extends FingerprintConfig>)Class.forName(config);
        final Map<String, Method> getters = new HashMap<String, Method>();
        final Map<String, Method> setters = new HashMap<String, Method>();
        for (final Method lMethod : fc.getMethods()) {
          if (lMethod.getName().startsWith(GET) || lMethod.getName().startsWith(IS) || lMethod.getName().startsWith(HAS)) {
            getters.put(lMethod.getName(), lMethod);
          }
          if (lMethod.getName().startsWith(SET)) {
            setters.put(lMethod.getName(), lMethod);
          }
        }
        final List<ConfigProperty> full = new ArrayList<ConfigProperty>();
        for (final String lKey : getters.keySet()) {
          String prop = lKey;
          if (prop.startsWith(GET) || prop.startsWith(HAS)) {
            prop = prop.substring(3);
          } else {
            // if (prop.startsWith(IS))
            prop = prop.substring(2);
          }
          PropertyDescriptor md = null;
          // check for bean property
          if (setters.containsKey(SET + prop)) {
            md = new PropertyDescriptor(prop.substring(0, 1).toLowerCase() + prop.substring(1), fc);
            md.setBound(true);
            md.setConstrained(true);
          }
          // check for list contrained property
          if (getters.containsKey(lKey + Chars.s) && (md != null)) {
            // found a list
            final Method ref = fc.getMethod(GET_INSTANCE, new Class[]{});
            final FingerprintConfig conf = (FingerprintConfig)ref.invoke(null, new Object[]{});
            final Method m = getters.get(lKey + Chars.s);
            md.setValue(ENUMERATION_VALUES, m.invoke(conf, new Object[]{}));
          }
          if (md != null) {
            full.add(new ConfigProperty(md, fc));
          }
        }
        Collections.sort(full, new ConfigPropertyComparator());
        return full.toArray();
      } catch (final ClassNotFoundException cnf) {
        cnf.printStackTrace();
      } catch (final IntrospectionException ex) {
        ex.printStackTrace();
      } catch (final NoSuchMethodException nsm) {
        nsm.printStackTrace();
      } catch (final IllegalAccessException ia) {
        ia.printStackTrace();
      } catch (final InvocationTargetException ite) {
        ite.printStackTrace();
      }
    } catch (final InstantiationException e) {
      e.printStackTrace();
    } catch (final IllegalAccessException e) {
      e.printStackTrace();
    } catch (final Exception t) {
      t.printStackTrace();
    }
    return new Object[0];
  }

  /**
   * @see org.eclipse.jface.viewers.IContentProvider#dispose()
   */
  public void dispose() {
    // Auto-generated method stub
  }

  /**
   * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
   */
  public void inputChanged(final Viewer pArg0, final Object pArg1, final Object pArg2) {
    // Auto-generated method stub
  }
}
