/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.gui.config;

import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;

import net.za.grasser.duplicate.fingerprint.configure.FingerprintConfig;
import net.za.grasser.duplicate.gui.properties.IProperty;
import net.za.grasser.duplicate.util.Strings;
import net.za.grasser.duplicate.util.res.Debug;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.widgets.Composite;

/**
 * This class ...
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.11 $
 */
public class ConfigProperty implements Comparable<ConfigProperty>, IProperty {
  /**
   * <code>GET_INSTANCE</code> ConfigProperty -
   */
  private static final String GET_INSTANCE = "getInstance"; //$NON-NLS-1$
  /**
   * <code>log</code> ConfigProperty -
   */
  private static final Logger log = Logger.getLogger(ConfigProperty.class);
  /**
   * <code>propertyDescriptor</code> ConfigProperty -
   */
  private PropertyDescriptor propertyDescriptor = null;
  /**
   * <code>beanInfoClass</code> ConfigProperty -
   */
  private Class< ? extends FingerprintConfig> beanInfoClass = null;
  /**
   * <code>cellEditor</code> ConfigProperty -
   */
  private CellEditor cellEditor = null;

  /**
   * Constructor
   * 
   * @param pd
   * @param bic
   */
  public ConfigProperty(final PropertyDescriptor pd, final Class< ? extends FingerprintConfig> bic) {
    propertyDescriptor = pd;
    beanInfoClass = bic;
  }

  /**
   * @return Class<? extends FingerprintConfig> - Returns the beanInfoClass.
   */
  public Class< ? extends FingerprintConfig> getBeanInfoClass() {
    return beanInfoClass;
  }

  /**
   * Sets the beanInfoClass to the value of pBeanInfoClass.
   * 
   * @param pBeanInfoClass Class<? extends FingerprintConfig> - The new value.
   */
  public void setBeanInfoClass(final Class< ? extends FingerprintConfig> pBeanInfoClass) {
    beanInfoClass = pBeanInfoClass;
  }

  /**
   * @return PropertyDescriptor - Returns the propertyDescriptor.
   */
  public PropertyDescriptor getPropertyDescriptor() {
    return propertyDescriptor;
  }

  /**
   * Sets the propertyDescriptor to the value of pPropertyDescriptor.
   * 
   * @param pPropertyDescriptor PropertyDescriptor - The new value.
   */
  public void setPropertyDescriptor(final PropertyDescriptor pPropertyDescriptor) {
    propertyDescriptor = pPropertyDescriptor;
  }

  /**
   * @param pO
   * @return 0 if the ConfigProperty are equal
   * @see java.lang.Comparable#compareTo(java.lang.Object)
   */
  public int compareTo(final ConfigProperty pO) {
    return propertyDescriptor.getDisplayName().compareTo(pO.getPropertyDescriptor().getDisplayName());
  }

  /**
   * @see net.za.grasser.duplicate.gui.properties.IProperty#getProperty()
   */
  public String getProperty() {
    return propertyDescriptor.getDisplayName();
  }

  /**
   * @see net.za.grasser.duplicate.gui.properties.IProperty#getValue()
   */
  public String getValue() {
    try {
      final Method ref = beanInfoClass.getMethod(GET_INSTANCE, new Class[]{});
      final FingerprintConfig config = (FingerprintConfig)ref.invoke(null, new Object[]{});
      final Method m = propertyDescriptor.getReadMethod();
      return m.invoke(config, new Object[]{}).toString();
    } catch (final Exception nsm) {
      log.error(Debug.getString(ConfigProperty.class, 1), nsm);
    }
    return Strings.EMPTY;
  }

  /**
   * @see net.za.grasser.duplicate.gui.properties.IProperty#setValue(java.lang.Object)
   */
  public void setValue(final Object pValue) {
    try {
      final Method ref = beanInfoClass.getMethod(GET_INSTANCE, new Class[]{});
      final FingerprintConfig config = (FingerprintConfig)ref.invoke(null, new Object[]{});
      final Method m = propertyDescriptor.getWriteMethod();
      m.getParameterTypes();
      // TODO cast value to the correct type
      m.invoke(config, new Object[]{
        pValue
      }).toString();
    } catch (final Exception nsm) {
      log.error(Debug.getString(ConfigProperty.class, 2), nsm);
    }
  }

  /**
   * @see net.za.grasser.duplicate.gui.properties.IProperty#getCellEditor(org.eclipse.swt.widgets.Composite, int)
   */
  public CellEditor getCellEditor(final Composite parent, final int style) {
    if (cellEditor == null) {
      // TODO create a CellEditor according to the data type
      cellEditor = new TextCellEditor(parent, style);
    }
    return cellEditor;
  }
}
