/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.gui.events;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.Widget;

/**
 * This class implements a singleton Listener that will listen for key pressed events and execute a SWT.Selected event on the mapped Widget. To use: attach the
 * listener to the
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.1 $
 */
public class HotKeyListener implements Listener {
  /**
   * <code>singleton</code> HotKeyListener -
   */
  private static HotKeyListener singleton = null;
  /**
   * <code>keyMap</code> HotKeyListener -
   */
  private Map<HotKey, Widget> keyMap = new HashMap<HotKey, Widget>();

  /**
   * Constructor
   */
  private HotKeyListener() {
    super();
  }

  /**
   * This method returns the singleton instance.
   * 
   * @return HotKeyListener
   */
  public static final HotKeyListener getInstance() {
    if (singleton == null) {
      singleton = new HotKeyListener();
    }
    return singleton;
  }

  /**
   * This method adds a key mapping from a hot key to a widget.
   * 
   * @param pStateMask
   * @param pKeyCode
   * @param pWidget
   */
  public void addMapping(final int pStateMask, final int pKeyCode, final Widget pWidget) {
    addMapping(new HotKey(pStateMask, pKeyCode), pWidget);
  }

  /**
   * This method adds a key mapping from a hot key to a widget.
   * 
   * @param pHot
   * @param pWidget
   */
  public void addMapping(final HotKey pHot, final Widget pWidget) {
    synchronized (keyMap) {
      keyMap.put(pHot, pWidget);
    }
  }

  /**
   * This method removes a key mapping
   * 
   * @param pHot
   */
  public void removeMapping(final HotKey pHot) {
    synchronized (keyMap) {
      keyMap.remove(pHot);
    }
  }

  /**
   * This method removes a key mapping
   * 
   * @param pStateMask
   * @param pKeyCode
   */
  public void removeMapping(final int pStateMask, final int pKeyCode) {
    removeMapping(new HotKey(pStateMask, pKeyCode));
  }

  /**
   * This method checks whether the widget is enabled. Since not all Widgets have an isEnabled function we need to use reflection.
   * 
   * @param widget
   * @return boolean
   */
  private static boolean isEnabled(Widget widget) {
    try {
      Method m = widget.getClass().getMethod("isEnabled", (Class[])null);
      return ((Boolean)m.invoke(widget)).booleanValue();
    } catch (SecurityException e) {
      return false;
    } catch (NoSuchMethodException e) {
      // if method does not exit return true
      return true;
    } catch (IllegalArgumentException e) {
      return false;
    } catch (IllegalAccessException e) {
      return false;
    } catch (InvocationTargetException e) {
      return false;
    }
  }

  /**
   * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
   */
  public void handleEvent(Event event) {
    if (event.stateMask == 262144) {
      if (event.keyCode == 97) {
        // Control A = select all
        if (event.widget instanceof Tree && (event.widget.getStyle() & SWT.MULTI) == SWT.MULTI) {
          ((Tree)event.widget).selectAll();
        } else if (event.widget instanceof Table && (event.widget.getStyle() & SWT.MULTI) == SWT.MULTI) {
          ((Table)event.widget).selectAll();
        }
      } else if (event.keyCode == 100) {
        // Control D = select none
        if (event.widget instanceof Tree) {
          ((Tree)event.widget).deselectAll();
        } else if (event.widget instanceof Table) {
          ((Table)event.widget).deselectAll();
        }
      }
    }
    HotKey eventKey = new HotKey(event.stateMask, event.keyCode);
    final Widget widget;
    synchronized (keyMap) {
      widget = keyMap.get(eventKey);
    }
    // if (widget != null && (event.widget != widget)) {
    if (widget != null && !widget.isDisposed() && isEnabled(widget)) {
      Event e2 = new Event();
      e2.widget = widget;
      e2.button = event.button;
      e2.character = event.character;
      e2.count = event.count;
      e2.data = event.data;
      e2.detail = event.detail;
      e2.display = event.display;
      e2.doit = true;
      e2.end = event.end;
      e2.gc = event.gc; // TODO where to get GC from?
      e2.height = event.height;
      e2.index = event.index;
      e2.item = event.item; // TODO same as e2.widget?
      e2.keyCode = event.keyCode;
      e2.start = event.start;
      e2.stateMask = event.stateMask;
      e2.text = event.text;
      e2.time = (int)(new Date().getTime() & 0xFFFFFFFFL);
      e2.type = SWT.Selection;
      e2.width = event.width;
      e2.x = event.x;
      e2.y = event.y;
      e2.widget.notifyListeners(e2.type, e2);
      event.doit = false;
    }
  }
}
