/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.gui.file;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import net.za.grasser.duplicate.file.FingerprintFile;
import net.za.grasser.duplicate.persist.DatabaseFactory;
import net.za.grasser.duplicate.util.Chars;
import net.za.grasser.duplicate.util.RegExs;
import net.za.grasser.duplicate.util.Strings;
import net.za.grasser.duplicate.util.res.Messages;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;

/**
 * This class provides delete, move and trash functionality to the whole project.
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.12 $
 */
public class FileUtils {
  /**
   * <code>shell</code> FileUtils -
   */
  private Shell shell;
  /**
   * <code>singleton</code> FileUtils -
   */
  private static FileUtils singleton = null;
  /**
   * <code>trashDir</code> FileUtils - save current move directory
   */
  String trashDir = null;

  /**
   * This class ...
   * 
   * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
   * @version $Revision: 1.12 $
   */
  public class Result {
    /**
     * <code>state</code> Result -
     */
    public boolean state = false;
    /**
     * <code>message</code> Result -
     */
    public String message = null;
  }

  /**
   * Constructor
   * 
   * @param pShell
   */
  private FileUtils(final Shell pShell) {
    super();
    shell = pShell;
  }

  /**
   * This method is the singleton access method
   * 
   * @param pShell
   * @return FileUtils
   */
  public static synchronized FileUtils getInstance(final Shell pShell) {
    if (singleton == null) {
      singleton = new FileUtils(pShell);
    }
    return singleton;
  }

  /**
   * This method will pop up a File selector box and move the file.
   * 
   * @param pFile
   * @return Result
   */
  public Result moveFile(final FingerprintFile pFile) {
    final FileDialog fileMover = new FileDialog(shell, SWT.SAVE);
    fileMover.setText(Messages.getString(FileUtils.class, 1));
    fileMover.setFilterPath(new File(pFile.getPath()).getParentFile().getPath());
    fileMover.setFileName(new File(pFile.getPath()).getName());
    // TODO get extensions/types from FingerprintFactory
    fileMover.setFilterNames(new String[]{
      Messages.getString(FileUtils.class, 2)
    });
    fileMover.setFilterExtensions(new String[]{
      RegExs.STAR_DOT_STAR
    });
    final File fFrom = new File(pFile.getPath());
    final Result r = new Result();
    final List<String> replace = new ArrayList<String>();
    replace.add(fFrom.getName());
    final String filename = fileMover.open();
    if (filename != null) {
      final File fTo = new File(filename);
      replace.add(fTo.getPath());
      if (fFrom.renameTo(fTo)) {
        r.state = true;
        DatabaseFactory.getDatabase().move(pFile, fTo);
        r.message = Messages.getString(FileUtils.class, 3, replace);
      } else {
        r.state = false;
        r.message = Messages.getString(FileUtils.class, 4, replace);
      }
    } else {
      r.state = false;
      r.message = Messages.getString(FileUtils.class, 5, replace);
    }
    return r;
  }

  /**
   * This method will ask for a trash directory and after that will move the files into that dir.
   * 
   * @param pFile
   * @return Result
   */
  public Result trashFile(final FingerprintFile pFile) {
    final FileDialog fileMover = new FileDialog(shell, SWT.SAVE);
    final Result r = new Result();
    final List<String> replace = new ArrayList<String>();
    replace.add(pFile.getName());
    fileMover.setText(Messages.getString(FileUtils.class, 6));
    if ((trashDir == null) || trashDir.equals(Strings.EMPTY)) {
      trashDir = new File(pFile.getPath()).getParentFile().getPath();
      fileMover.setFilterPath(trashDir);
      fileMover.setFileName(new File(pFile.getPath()).getName());
      fileMover.setFilterNames(new String[]{
        Messages.getString(FileUtils.class, 2)
      });
      fileMover.setFilterExtensions(new String[]{
        RegExs.STAR_DOT_STAR
      });
      final String filename = fileMover.open();
      if (filename != null) {
        trashDir = fileMover.getFilterPath();
      } else {
        r.state = false;
        r.message = Messages.getString(FileUtils.class, 7, replace);
        return r;
      }
    }
    final File fFrom = new File(pFile.getPath());
    String name = pFile.getName();
    String ext = Strings.EMPTY;
    if (name.indexOf(Chars.DOT) >= 0) {
      ext = name.substring(name.indexOf(Chars.DOT));
      name = name.substring(0, name.indexOf(Chars.DOT));
    }
    File fTo = new File(trashDir + File.separator + name + ext);
    int count = 1;
    while (fTo.exists()) {
      fTo = new File(trashDir + File.separator + name + Chars.USCORE + String.valueOf(count++) + ext);
    }
    if (fFrom.renameTo(fTo)) {
      r.state = true;
      replace.add(fTo.getPath());
      DatabaseFactory.getDatabase().move(pFile, fTo);
      r.message = Messages.getString(FileUtils.class, 8, replace);
    } else {
      r.state = false;
      r.message = Messages.getString(FileUtils.class, 9, replace);
    }
    return r;
  }

  /**
   * This method deletes the file and refreshes the tree
   * 
   * @param pFile
   * @return Result
   */
  public Result deleteFile(final FingerprintFile pFile) {
    final File f = new File(pFile.getPath());
    final List<String> replace = new ArrayList<String>();
    replace.add(f.getName());
    final Result r = new Result();
    if (f.delete()) {
      r.state = true;
      DatabaseFactory.getDatabase().delete(pFile);
      r.message = Messages.getString(FileUtils.class, 10, replace);
    } else {
      r.state = false;
      r.message = Messages.getString(FileUtils.class, 11, replace);
    }
    return r;
  }
}
