/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.gui.info;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import net.za.grasser.duplicate.compare.Comparer;
import net.za.grasser.duplicate.file.FingerprintFile;
import net.za.grasser.duplicate.fingerprint.AbstractFingerprint;
import net.za.grasser.duplicate.fingerprint.FdmfFingerprint;
import net.za.grasser.duplicate.fingerprint.FingerprintFactory;
import net.za.grasser.duplicate.gui.IDisplay;
import net.za.grasser.duplicate.gui.StringSorter;
import net.za.grasser.duplicate.gui.events.HotKeyListener;
import net.za.grasser.duplicate.gui.file.FileUtils;
import net.za.grasser.duplicate.persist.DatabaseFactory;
import net.za.grasser.duplicate.util.Chars;
import net.za.grasser.duplicate.util.Files;
import net.za.grasser.duplicate.util.res.Messages;

import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

import com.cloudgarden.resource.SWTResourceManager;

/**
 * This class ...
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.21 $
 */
public class DoubleInfo extends Composite {
  /**
   * <code>tableViewerLeft</code> DoubleInfo -
   */
  TableViewer tableViewerLeft;
  /**
   * <code>tableViewerRight</code> DoubleInfo -
   */
  TableViewer tableViewerRight;
  /**
   * <code>toolBarMiddle</code> DoubleInfo -
   */
  private ToolBar toolBarMiddle;
  /**
   * <code>toolItemUnequal</code> DoubleInfo -
   */
  private ToolItem toolItemUnequal;
  /**
   * <code>toolItemDeleteBoth</code> DoubleInfo -
   */
  private ToolItem toolItemDeleteBoth;
  /**
   * <code>toolItemTrashBoth</code> DoubleInfo -
   */
  private ToolItem toolItemTrashBoth;
  /**
   * <code>toolItemMoveBoth</code> DoubleInfo -
   */
  private ToolItem toolItemMoveBoth;
  /**
   * <code>toolItemCopyInfoRight</code> DoubleInfo -
   */
  private ToolItem toolItemCopyInfoRight;
  /**
   * <code>toolItemCopyInfoLeft</code> DoubleInfo -
   */
  private ToolItem toolItemCopyInfoLeft;
  /**
   * <code>fire</code> DoubleInfo -
   */
  boolean fire = true;
  /**
   * <code>left</code> DoubleInfo -
   */
  FingerprintFile left;
  /**
   * <code>right</code> DoubleInfo -
   */
  FingerprintFile right;
  /**
   * <code>listener</code> DoubleInfo - 
   */
  private static final HotKeyListener listener = HotKeyListener.getInstance();

  /**
   * Constructor
   * 
   * @param pArg0
   * @param pArg1
   */
  public DoubleInfo(final Composite pArg0, final int pArg1) {
    super(pArg0, pArg1);
    getTableViewerLeft();
    getToolBarMiddle();
    getTableViewerRight();
  }

  /**
   * This method translates the keys of the map.
   * 
   * @param source
   * @return Map<String,String>
   */
  private Map<String, String> translate(final Map<String, String> source) {
    final Map<String, String> dest = new HashMap<String, String>();
    for (final String lString : source.keySet()) {
      dest.put(Messages.getString(DoubleInfo.class, lString.replace(Chars.SPACE, Chars.USCORE).replace(Chars.EQ, Chars.DASH)),
        source.get(lString));
    }
    return dest;
  }

  /**
   * This method will display the fingerprint files additional info in two lists.
   * 
   * @param pLeft - can be null
   * @param pRight - can be null
   */
  public void display(final FingerprintFile pLeft, final FingerprintFile pRight) {
    toolItemCopyInfoLeft.setEnabled(false);
    toolItemCopyInfoRight.setEnabled(false);
    toolItemDeleteBoth.setEnabled(false);
    toolItemMoveBoth.setEnabled(false);
    toolItemTrashBoth.setEnabled(false);
    toolItemUnequal.setEnabled(false);
    left = pLeft;
    right = pRight;
    boolean bothValid = true;
    if (left == null) {
      tableViewerLeft.setInput(null);
      bothValid = false;
    } else {
      // translate info field names to Local string
      tableViewerLeft.setInput(translate(left.getInfo()));
      final List<Class< ? extends AbstractFingerprint>> s = FingerprintFactory.getFingerprintTypes(left.getExtension());
      final Iterator<Class< ? extends AbstractFingerprint>> it = s.iterator();
      while (it.hasNext()) {
        final Class< ? > cls = it.next();
        if (cls.getName().indexOf(FdmfFingerprint.class.getSimpleName()) >= 0) {
          toolItemCopyInfoLeft.setEnabled(true);
        }
      }
    }
    if (right == null) {
      tableViewerRight.setInput(null);
      bothValid = false;
    } else {
      // translate info field names to Local string
      tableViewerRight.setInput(translate(right.getInfo()));
      final List<Class< ? extends AbstractFingerprint>> s = FingerprintFactory.getFingerprintTypes(right.getExtension());
      final Iterator<Class< ? extends AbstractFingerprint>> it = s.iterator();
      while (it.hasNext()) {
        final Class< ? > cls = it.next();
        if (cls.getName().indexOf(FdmfFingerprint.class.getSimpleName()) >= 0) {
          toolItemCopyInfoRight.setEnabled(true);
        }
      }
    }
    if (bothValid) {
      colorItems();
      toolItemDeleteBoth.setEnabled(true);
      toolItemMoveBoth.setEnabled(true);
      toolItemTrashBoth.setEnabled(true);
      toolItemUnequal.setEnabled(true);
    }
    tableViewerLeft.refresh();
    tableViewerRight.refresh();
  }

  /**
   * This method ...
   */
  @SuppressWarnings("unchecked")
  private void colorItems() {
    for (int i = tableViewerLeft.getTable().getItemCount() - 1; i >= 0; i--) {
      final TableItem litem = tableViewerLeft.getTable().getItem(i);
      final Map.Entry ldata = (Map.Entry)litem.getData();
      for (int j = tableViewerRight.getTable().getItemCount() - 1; j >= 0; j--) {
        final TableItem ritem = tableViewerRight.getTable().getItem(j);
        final Map.Entry rdata = (Map.Entry)ritem.getData();
        if (ldata.getKey().equals(rdata.getKey())) {
          if (!compare(ldata.getValue(), rdata.getValue())) {
            litem.setBackground(SWTResourceManager.getColor(255, 255, 102));
            ritem.setBackground(SWTResourceManager.getColor(255, 255, 102));
          }
          break;
        }
      }
    }
  }

  /**
   * This method returns true if the two objects are the same. Cuts file string to the file name only.
   * 
   * @param l
   * @param r
   * @return boolean
   */
  private boolean compare(final Object l, final Object r) {
    String le = l.toString(), ri = r.toString();
    if (le.indexOf(File.separator) >= 0) {
      le = le.substring(le.lastIndexOf(File.separator) + 1);
    }
    if (ri.indexOf(File.separator) >= 0) {
      ri = ri.substring(ri.lastIndexOf(File.separator) + 1);
    }
    return le.equals(ri);
  }

  /**
   * @return TableViewer - Returns the tableViewerLeft.
   */
  public TableViewer getTableViewerLeft() {
    if (tableViewerLeft == null) {
      tableViewerLeft = new TableViewer(this, SWT.BORDER | SWT.FULL_SELECTION | SWT.SINGLE);
      final FormData tableViewerLeftLData = new FormData();
      tableViewerLeftLData.width = 67;
      tableViewerLeftLData.height = 63;
      tableViewerLeftLData.left = new FormAttachment(0, 1000, 0);
      tableViewerLeftLData.right = new FormAttachment(500, 1000, -14);
      tableViewerLeftLData.top = new FormAttachment(0, 1000, 0);
      tableViewerLeftLData.bottom = new FormAttachment(1000, 1000, 0);
      tableViewerLeft.getControl().setLayoutData(tableViewerLeftLData);
      tableViewerLeft.getTable().setHeaderVisible(true);
      final TableColumn column1 = new TableColumn(tableViewerLeft.getTable(), SWT.LEFT);
      column1.setText(Messages.getString(DoubleInfo.class, 1));
      column1.setWidth(100);
      final TableColumn column2 = new TableColumn(tableViewerLeft.getTable(), SWT.RIGHT);
      column2.setText(Messages.getString(DoubleInfo.class, 2));
      column2.setWidth(120);
      tableViewerLeft.setContentProvider(new MapTableContentProvider());
      tableViewerLeft.setLabelProvider(new PropertyLabelProvider());
      tableViewerLeft.setSorter(new StringSorter());
      tableViewerLeft.addSelectionChangedListener(new ISelectionChangedListener() {
        public void selectionChanged(final SelectionChangedEvent evt) {
          if (fire) {
            tableViewerLeftSelectionChanged(evt);
          }
        }
      });
    }
    return tableViewerLeft;
  }

  /**
   * @return TableViewer - Returns the tableViewerRight.
   */
  public TableViewer getTableViewerRight() {
    if (tableViewerRight == null) {
      tableViewerRight = new TableViewer(this, SWT.BORDER | SWT.FULL_SELECTION | SWT.SINGLE);
      final FormData tableViewerRightLData = new FormData();
      tableViewerRightLData.width = 67;
      tableViewerRightLData.height = 63;
      tableViewerRightLData.left = new FormAttachment(500, 1000, 14);
      tableViewerRightLData.right = new FormAttachment(1000, 1000, 0);
      tableViewerRightLData.top = new FormAttachment(0, 1000, 0);
      tableViewerRightLData.bottom = new FormAttachment(1000, 1000, 0);
      tableViewerRight.getControl().setLayoutData(tableViewerRightLData);
      tableViewerRight.getTable().setHeaderVisible(true);
      final TableColumn column1 = new TableColumn(tableViewerRight.getTable(), SWT.LEFT);
      column1.setText(Messages.getString(DoubleInfo.class, 1));
      column1.setWidth(100);
      final TableColumn column2 = new TableColumn(tableViewerRight.getTable(), SWT.RIGHT);
      column2.setText(Messages.getString(DoubleInfo.class, 2));
      column2.setWidth(120);
      tableViewerRight.setContentProvider(new MapTableContentProvider());
      tableViewerRight.setLabelProvider(new PropertyLabelProvider());
      tableViewerRight.setSorter(new StringSorter());
      tableViewerRight.addSelectionChangedListener(new ISelectionChangedListener() {
        public void selectionChanged(final SelectionChangedEvent evt) {
          if (fire) {
            tableViewerRightSelectionChanged(evt);
          }
        }
      });
    }
    return tableViewerRight;
  }

  /**
   * @return ToolBar
   */
  public ToolBar getToolBarMiddle() {
    if (toolBarMiddle == null) {
      toolBarMiddle = new ToolBar(this, SWT.WRAP | SWT.BORDER | SWT.VERTICAL);
      final FormData toolBar3LData = new FormData();
      toolBar3LData.width = 27;
      toolBar3LData.height = 112;
      toolBar3LData.left = new FormAttachment(500, 1000, -14);
      toolBar3LData.right = new FormAttachment(500, 1000, 14);
      toolBar3LData.top = new FormAttachment(0, 1000, 0);
      toolBar3LData.bottom = new FormAttachment(1000, 1000, 0);
      toolBarMiddle.setLayoutData(toolBar3LData);
      getToolItemUnequal();
      getToolItemDeleteBoth();
      getToolItemTrashBoth();
      getToolItemMoveBoth();
      getToolItemCopyInfoRight();
      getToolItemCopyInfoLeft();
    }
    return toolBarMiddle;
  }

  /**
   * @return ToolItem - Returns the toolItemCopyInfoLeft.
   */
  public ToolItem getToolItemCopyInfoLeft() {
    if (toolItemCopyInfoLeft == null) {
      toolItemCopyInfoLeft = new ToolItem(toolBarMiddle, SWT.PUSH);
      listener.addMapping(SWT.CONTROL, 44, toolItemCopyInfoLeft);
      toolItemCopyInfoLeft.setImage(SWTResourceManager.getImage(Files.ICONS_COPY_INFO_LEFT_GIF));
      toolItemCopyInfoLeft.setToolTipText(Messages.getString(DoubleInfo.class, 3));
      toolItemCopyInfoLeft.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(final SelectionEvent evt) {
          // toolItemCopyInfoLeftWidgetSelected(evt);
        }
      });
    }
    return toolItemCopyInfoLeft;
  }

  /**
   * @return ToolItem - Returns the toolItemCopyInfoRight.
   */
  public ToolItem getToolItemCopyInfoRight() {
    if (toolItemCopyInfoRight == null) {
      toolItemCopyInfoRight = new ToolItem(toolBarMiddle, SWT.PUSH);
      listener.addMapping(SWT.CONTROL, 46, toolItemCopyInfoRight);
      toolItemCopyInfoRight.setImage(SWTResourceManager.getImage(Files.ICONS_COPY_INFO_RIGHT_GIF));
      toolItemCopyInfoRight.setToolTipText(Messages.getString(DoubleInfo.class, 4));
      toolItemCopyInfoRight.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(final SelectionEvent evt) {
          // toolItemCopyInfoRightWidgetSelected(evt);
        }
      });
    }
    return toolItemCopyInfoRight;
  }

  /**
   * @return ToolItem - Returns the toolItemDeleteBoth.
   */
  public ToolItem getToolItemDeleteBoth() {
    if (toolItemDeleteBoth == null) {
      toolItemDeleteBoth = new ToolItem(toolBarMiddle, SWT.PUSH);
      listener.addMapping(SWT.CONTROL, 98, toolItemDeleteBoth);
      toolItemDeleteBoth.setImage(SWTResourceManager.getImage(Files.ICONS_REMOVE2_GIF));
      toolItemDeleteBoth.setToolTipText(Messages.getString(DoubleInfo.class, 5));
      toolItemDeleteBoth.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(final SelectionEvent evt) {
          toolItemDeleteBothWidgetSelected(evt);
        }
      });
    }
    return toolItemDeleteBoth;
  }

  /**
   * This method deletes both files permanantly.
   * 
   * @param pEvt
   */
  protected void toolItemDeleteBothWidgetSelected(final SelectionEvent pEvt) {
    if ((left == null) || (right == null)) {
      return;
    }
    final FileUtils.Result l = FileUtils.getInstance(getShell()).deleteFile(left);
    if (l.state) {
      Comparer.getInstance().removeFile(left);
    }
    final FileUtils.Result r = FileUtils.getInstance(getShell()).deleteFile(right);
    if (r.state) {
      Comparer.getInstance().removeFile(right);
    }
    final IDisplay d = locateDisplay();
    if (d != null) {
      final List<String> replace = new ArrayList<String>();
      replace.add(l.message);
      replace.add(r.message);
      d.setStatus(Messages.getString(DoubleInfo.class, 10, replace));
      if (l.state || r.state) {
        d.refreshResults();
      }
    }
  }

  /**
   * @return ToolItem - Returns the toolItemMoveBoth.
   */
  public ToolItem getToolItemMoveBoth() {
    if (toolItemMoveBoth == null) {
      toolItemMoveBoth = new ToolItem(toolBarMiddle, SWT.PUSH);
      listener.addMapping(SWT.CONTROL, 109, toolItemMoveBoth);
      toolItemMoveBoth.setImage(SWTResourceManager.getImage(Files.ICONS_MOVE2_GIF));
      toolItemMoveBoth.setToolTipText(Messages.getString(DoubleInfo.class, 6));
      toolItemMoveBoth.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(final SelectionEvent evt) {
          toolItemMoveBothWidgetSelected(evt);
        }
      });
    }
    return toolItemMoveBoth;
  }

  /**
   * This method moves both files individually.
   * 
   * @param pEvt
   */
  protected void toolItemMoveBothWidgetSelected(final SelectionEvent pEvt) {
    if ((left == null) || (right == null)) {
      return;
    }
    final FileUtils.Result l = FileUtils.getInstance(getShell()).moveFile(left);
    if (l.state) {
      Comparer.getInstance().removeFile(left);
    }
    final FileUtils.Result r = FileUtils.getInstance(getShell()).moveFile(right);
    if (r.state) {
      Comparer.getInstance().removeFile(right);
    }
    final IDisplay d = locateDisplay();
    if (d != null) {
      final List<String> replace = new ArrayList<String>();
      replace.add(l.message);
      replace.add(r.message);
      d.setStatus(Messages.getString(DoubleInfo.class, 10, replace));
      if (l.state || r.state) {
        d.refreshResults();
      }
    }
  }

  /**
   * @return ToolItem - Returns the toolItemTrashBoth.
   */
  public ToolItem getToolItemTrashBoth() {
    if (toolItemTrashBoth == null) {
      toolItemTrashBoth = new ToolItem(toolBarMiddle, SWT.PUSH);
      listener.addMapping(SWT.CONTROL, 110, toolItemTrashBoth);
      toolItemTrashBoth.setImage(SWTResourceManager.getImage(Files.ICONS_TRASH_CAN2_GIF));
      toolItemTrashBoth.setToolTipText(Messages.getString(DoubleInfo.class, 7));
      toolItemTrashBoth.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(final SelectionEvent evt) {
          toolItemTrashBothWidgetSelected(evt);
        }
      });
    }
    return toolItemTrashBoth;
  }

  /**
   * This method moves both files tot he temporary trash bin.
   * 
   * @param pEvt
   */
  protected void toolItemTrashBothWidgetSelected(final SelectionEvent pEvt) {
    if ((left == null) || (right == null)) {
      return;
    }
    final FileUtils.Result l = FileUtils.getInstance(getShell()).trashFile(left);
    if (l.state) {
      Comparer.getInstance().removeFile(left);
    }
    final FileUtils.Result r = FileUtils.getInstance(getShell()).trashFile(right);
    if (r.state) {
      Comparer.getInstance().removeFile(right);
    }
    final IDisplay d = locateDisplay();
    if (d != null) {
      final List<String> replace = new ArrayList<String>();
      replace.add(l.message);
      replace.add(r.message);
      d.setStatus(Messages.getString(DoubleInfo.class, 10, replace));
      if (l.state || r.state) {
        d.refreshResults();
      }
    }
  }

  /**
   * @return ToolItem - Returns the toolItemUnequal.
   */
  public ToolItem getToolItemUnequal() {
    if (toolItemUnequal == null) {
      toolItemUnequal = new ToolItem(toolBarMiddle, SWT.PUSH);
      listener.addMapping(SWT.CONTROL, 120, toolItemUnequal);
      toolItemUnequal.setImage(SWTResourceManager.getImage(Files.ICONS_UNEQUAL_GIF));
      toolItemUnequal.setToolTipText(Messages.getString(DoubleInfo.class, 8));
      toolItemUnequal.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(final SelectionEvent evt) {
          toolItemUnequalWidgetSelected(evt);
        }
      });
    }
    return toolItemUnequal;
  }

  /**
   * This method marks the two files as not duplicate and saves the fact in the database.
   * 
   * @param pEvt
   */
  protected void toolItemUnequalWidgetSelected(final SelectionEvent pEvt) {
    if ((left == null) || (right == null)) {
      return;
    }
    DatabaseFactory.getDatabase().setNotDuplicate(left, right);
    Comparer.getInstance().moveFileUp(right);
    final IDisplay d = locateDisplay();
    if (d != null) {
      d.setStatus(Messages.getString(DoubleInfo.class, 9));
      d.refreshResults();
    }
  }

  /**
   * This method ...
   * 
   * @return IDisplay
   */
  private IDisplay locateDisplay() {
    Composite p = this;
    while (p.getParent() != null) {
      if (p instanceof IDisplay) {
        return (IDisplay)p;
      }
      p = p.getParent();
    }
    return null;
  }

  /**
   * This method synchronizes the right selection with the left
   * 
   * @param pEvt
   */
  @SuppressWarnings("unchecked")
  protected void tableViewerLeftSelectionChanged(final SelectionChangedEvent pEvt) {
    final IStructuredSelection selection = (IStructuredSelection)pEvt.getSelection();
    if (selection.isEmpty()) {
      return;
    }
    final Map.Entry<String, String> sel = (Map.Entry<String, String>)selection.getFirstElement();
    final String sKey = sel.getKey();
    final Map<String, String> m = (Map<String, String>)tableViewerRight.getInput();
    boolean found = false;
    for (final Map.Entry<String, String> lEntry : m.entrySet()) {
      if (sKey.equals(lEntry.getKey())) {
        found = true;
        fire = false;
        tableViewerRight.setSelection(new StructuredSelection(lEntry));
        fire = true;
        break;
      }
    }
    if (!found) {
      tableViewerRight.setSelection(null);
    }
  }

  /**
   * This method synchronizes the left selection with the right
   * 
   * @param pEvt
   */
  @SuppressWarnings("unchecked")
  protected void tableViewerRightSelectionChanged(final SelectionChangedEvent pEvt) {
    final IStructuredSelection selection = (IStructuredSelection)pEvt.getSelection();
    if (selection.isEmpty()) {
      return;
    }
    final Map.Entry<String, String> sel = (Map.Entry<String, String>)selection.getFirstElement();
    final String sKey = sel.getKey();
    final Map<String, String> m = (Map<String, String>)tableViewerLeft.getInput();
    boolean found = false;
    for (final Map.Entry<String, String> lEntry : m.entrySet()) {
      if (sKey.equals(lEntry.getKey())) {
        found = true;
        fire = false;
        tableViewerLeft.setSelection(new StructuredSelection(lEntry));
        fire = true;
        break;
      }
    }
    if (!found) {
      tableViewerLeft.setSelection(null);
    }
  }
}
