/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.persist;

import net.za.grasser.duplicate.fingerprint.AbstractFingerprint;

/**
 * This class is used to return the comparrison status of two fingerprints.
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.5 $
 */
public class CompareStatus {
  /**
   * <code>NOT_COMPARED</code> CompareStatus -
   */
  public static final float NOT_COMPARED = -2.0f;
  /**
   * <code>COMPARED_NOT_DUPLICATE</code> CompareStatus -
   */
  public static final float COMPARED_NOT_DUPLICATE = -1.0f;
  /**
   * <code>file1</code> CompareStatus -
   */
  private AbstractFingerprint fingerprint1 = null;
  /**
   * <code>file2</code> CompareStatus -
   */
  private AbstractFingerprint fingerprint2 = null;
  /**
   * <code>similarity</code> CompareStatus -
   */
  private float similarity = NOT_COMPARED;

  /**
   * Constructor
   * 
   * @param pFingerprint1
   * @param pFingerprint2
   */
  public CompareStatus(final AbstractFingerprint pFingerprint1, final AbstractFingerprint pFingerprint2) {
    super();
    fingerprint1 = pFingerprint1;
    fingerprint2 = pFingerprint2;
  }

  /**
   * @return double - Returns the similarity.
   */
  public float getSimilarity() {
    return similarity;
  }

  /**
   * Sets the similarity to the value of pSimilarity.
   * 
   * @param pSimilarity float - The new value.
   */
  public void setSimilarity(final float pSimilarity) {
    similarity = pSimilarity;
  }

  /**
   * @return AbstractFingerprint - Returns the fingerprint1.
   */
  public AbstractFingerprint getFingerprint1() {
    return fingerprint1;
  }

  /**
   * @return AbstractFingerprint - Returns the fingerprint2.
   */
  public AbstractFingerprint getFingerprint2() {
    return fingerprint2;
  }
}
