/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.persist;

import java.io.File;
import java.util.List;

import net.za.grasser.duplicate.file.FingerprintFile;

/**
 * This class defines the database instance
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.14 $
 */
public interface Database {
  /**
   * This method connects to the database.
   * 
   * @return boolean
   */
  public abstract boolean connect();

  /**
   * This method checks whether the database is connected
   * 
   * @return boolean
   */
  public abstract boolean isConnected();

  /**
   * This method closes the database
   */
  public abstract void close();

  /**
   * Retrieve the details for a <code>FingerprintFile</code>.
   * 
   * @param pFf a FingerprintFile
   * @return boolean
   */
  public abstract boolean get(final FingerprintFile pFf);

  /**
   * Store all details for a <code>FingerprintFile</code>.
   * 
   * @param pFf a FingerprintFile
   */
  public abstract void put(final FingerprintFile pFf);

  /**
   * This method deletes the file from the database.
   * 
   * @param pFf
   */
  public abstract void delete(final FingerprintFile pFf);

  /**
   * This method moves the file to a different location.
   * 
   * @param pFf
   * @param pPath
   */
  public abstract void move(final FingerprintFile pFf, File pPath);

  /**
   * Retrieve info for a <code>FingerprintFile</code> and store it in the <code>FingerprintFile</code>. Not done in <code>get</code> for performance and
   * space reasons.
   * 
   * @param pFf a FingerprintFile
   * @return FingerprintFile
   */
  public abstract boolean getInfo(final FingerprintFile pFf);

  /**
   * Store info for a <code>FingerprintFile</code>. Not done in <code>put</code> for performance reasons.
   * 
   * @param pFf a FingerprintFile
   */
  public abstract void putInfo(final FingerprintFile pFf);

  /**
   * This method checks whether the two files are <b>not</b> duplicates of one another.
   * 
   * @param pFf1
   * @param pFf2
   * @deprecated
   * @return boolean - true if the files are in the list of files that are not duplicates.
   */
  @Deprecated
  public abstract boolean isNotDuplicate(final FingerprintFile pFf1, final FingerprintFile pFf2);

  /**
   * This method marks the two files as not duplicate of each other. This will be used for the case where the fingerprints indicate a duplicate, but the filess
   * are not really the same. I.e. a false positive.
   * 
   * @param pFf1
   * @param pFf2
   */
  public abstract void setNotDuplicate(final FingerprintFile pFf1, final FingerprintFile pFf2);

  /**
   * This method checks whether the two files have been compared before and were marked duplicates.
   * 
   * @param pFf1
   * @param pFf2
   * @return List&lt;CompareStatus&gt;
   */
  public abstract List<CompareStatus> getSimilarity(final FingerprintFile pFf1, final FingerprintFile pFf2);

  /**
   * This method saves the fingerprint comparison result.
   * 
   * @param pStatus
   */
  public abstract void putSimilarity(final List<CompareStatus> pStatus);

  /**
   * This method returns the name of the connected database.
   * 
   * @return String
   */
  public abstract String getName();

  /**
   * This method returns the version of the connected database.
   * 
   * @return String
   */
  public abstract String getVersion();
}