/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.persist;

import java.util.Locale;
import java.util.ResourceBundle;

import net.za.grasser.duplicate.Configure;
import net.za.grasser.duplicate.util.Files;
import net.za.grasser.duplicate.util.Strings;
import net.za.grasser.duplicate.util.res.SQLCommands;

/**
 * This class ...
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.24 $
 */
public class FileDatabase extends SQLDatabase {
  /**
   * <code>SAVE_FILE</code> FileDatabase -
   */
  private static final String SAVE_FILE = "save file"; //$NON-NLS-1$
  /**
   * <code>H2_DRIVER</code> FileDatabase - H2 driver name
   */
  private static final String H2_DRIVER = "org.h2.Driver"; //$NON-NLS-1$
  /**
   * <code>H2_FILE</code> FileDatabase - start of H2 JDBC connection URL for file access
   */
  private static final String H2_FILE = "jdbc:h2:file:"; //$NON-NLS-1$
  /**
   * <code>H2_ADMIN</code> FileDatabase - H2 system administrator
   */
  private static final String H2_ADMIN = "sa"; //$NON-NLS-1$
  static {
    Configure.load();
    // Load a database driver
    final String filename = Configure.getProperty(SAVE_FILE, Files.DATABASE_DAT, FileDatabase.class.getName());
    driver = H2_DRIVER;
    url = H2_FILE + filename;
    user = H2_ADMIN;
    password = Strings.EMPTY;
  }

  /**
   * This class will replace the standard SQLCommnds so that the H2 commands can be separated from the normal db commands.
   * 
   * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
   * @version $Revision: 1.24 $
   */
  protected class H2Commands extends SQLCommands {
    /**
     * <code>H2_VERSION</code> H2Commands - 1.0.67
     */
    private static final String H2_VERSION = "1.0.67"; //$NON-NLS-1$
    /**
     * <code>H2_LANGUAGE</code> H2Commands - h2
     */
    private static final String H2_LANGUAGE = "h2"; //$NON-NLS-1$

    /**
     * Constructor
     */
    public H2Commands() {
      super();
      RESOURCE_BUNDLE = ResourceBundle.getBundle(Files.SQL_BUNDLE_NAME, new Locale(H2_LANGUAGE, H2_VERSION));
    }
  }

  /**
   * Constructor
   */
  FileDatabase() {
    super();
  }

  /**
   * @see net.za.grasser.duplicate.persist.SQLDatabase#connect()
   */
  @Override
  public boolean connect() {
    sqlCommands = new H2Commands();
    return super.connect();
  }
}
