/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.persist;

import java.io.File;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;

import jonelo.sugar.util.Base64;
import net.za.grasser.duplicate.Configure;
import net.za.grasser.duplicate.file.DirectoryInfo;
import net.za.grasser.duplicate.file.FingerprintFile;
import net.za.grasser.duplicate.file.Status;
import net.za.grasser.duplicate.fingerprint.AbstractFingerprint;
import net.za.grasser.duplicate.util.Chars;
import net.za.grasser.duplicate.util.HexArray;
import net.za.grasser.duplicate.util.Strings;
import net.za.grasser.duplicate.util.res.Debug;
import net.za.grasser.duplicate.util.res.SQLCommands;

import org.apache.log4j.Logger;

/**
 * This class ...
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.44 $
 */
public class SQLDatabase implements Database {
  /**
   * This class is a helper to get the file id and status from the db simultaneously.
   * 
   * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
   * @version $Revision: 1.44 $
   */
  class IDStatus {
    /**
     * <code>fileid</code> IDStatus -
     */
    long fileid = -1;
    /**
     * <code>status</code> IDStatus -
     */
    Status status = Status.FILE_UNKNOWN;
  }
  /**
   * This class keeps the Database connection alive for as long as the program is running.
   * 
   * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
   * @version $Revision: 1.44 $
   */
  class KeepAlive implements Runnable {
    /**
     * <code>running</code> KeepAlive -
     */
    public AtomicBoolean running = new AtomicBoolean(true);

    /**
     * @see java.lang.Runnable#run()
     */
    public void run() {
      try {
        while (running.get()) {
          // sleep 1 hour
          Thread.sleep(3600000);
          checkAlive();
        }
      } catch (final InterruptedException e) {
        Thread.currentThread().interrupt();
        running.set(false);
      }
    }

    /**
     * This method stops the keep alive thread.
     */
    public void stop() {
      running.set(false);
    }
  }

  /**
   * <code>COL_FILEINFOTYPEID</code> SQLDatabase -
   */
  private static final String COL_FILEINFOTYPEID = "colFITI"; //$NON-NLS-1$
  /**
   * <code>COL_FILENAMEID</code> SQLDatabase -
   */
  private static final String COL_FILENAMEID = "colFNI"; //$NON-NLS-1$
  /**
   * <code>COL_FINGERPRINTTYPEID</code> SQLDatabase -
   */
  private static final String COL_FINGERPRINTTYPEID = "colFPTI"; //$NON-NLS-1$
  /**
   * <code>COL_FINGERPRINTID</code> SQLDatabase -
   */
  private static final String COL_FINGERPRINTID = "colFPI"; //$NON-NLS-1$
  /**
   * <code>conn</code> SQLDatabase - Database connection
   */
  private static Connection conn = null;
  /**
   * <code>DEFAULT_DATABASE</code> SQLDatabase -
   */
  private static final String DEFAULT_DATABASE = "jdbc:mysql://localhost/dump3"; //$NON-NLS-1$
  /**
   * <code>DEFAULT_JDBC_DRIVER</code> SQLDatabase -
   */
  private static final String DEFAULT_JDBC_DRIVER = "com.mysql.jdbc.Driver"; //$NON-NLS-1$
  /**
   * <code>DEFAULT_PASSWORD</code> SQLDatabase -
   */
  private static final String DEFAULT_PASSWORD = "password"; //$NON-NLS-1$
  /**
   * <code>DEFAULT_USER</code> SQLDatabase -
   */
  private static final String DEFAULT_USER = "root"; //$NON-NLS-1$
  /**
   * <code>driver</code> SQLDatabase -
   */
  protected static String driver = null;
  /**
   * <code>DRIVER</code> SQLDatabase -
   */
  private static final String DRIVER = "driver"; //$NON-NLS-1$
  /**
   * <code>fingerprinttypes</code> SQLDatabase -
   */
  private static HashMap<String, Integer> fingerprintTypes = null;
  /**
   * <code>infoTypes</code> SQLDatabase -
   */
  private static HashMap<String, Integer> infoTypes = null;
  /**
   * <code>log</code> SQLDatabase - Logger
   */
  private static final Logger log = Logger.getLogger(SQLDatabase.class);
  /**
   * <code>password</code> SQLDatabase -
   */
  protected static String password = null;
  /**
   * <code>PASSWORD</code> SQLDatabase -
   */
  private static final String PASSWORD = "password"; //$NON-NLS-1$
  /**
   * <code>stmtCheckAlive</code> SQLDatabase -
   */
  private static PreparedStatement stmtCheckAlive = null;
  /**
   * <code>stmtDeleteDuplicate</code> SQLDatabase -
   */
  private static PreparedStatement stmtDeleteDuplicate = null;
  /**
   * <code>stmtDeleteFile</code> SQLDatabase -
   */
  private static PreparedStatement stmtDeleteFile = null;
  /**
   * <code>stmtDeleteFingerprints</code> SQLDatabase -
   */
  private static PreparedStatement stmtDeleteFingerprints = null;
  /**
   * <code>stmtDeleteInfo</code> SQLDatabase -
   */
  private static PreparedStatement stmtDeleteInfo = null;
  /**
   * <code>stmtFindSimilarity</code> SQLDatabase -
   */
  private static PreparedStatement stmtFindSimilarity = null;
  /**
   * <code>stmtGetFileId</code> SQLDatabase -
   */
  private static PreparedStatement stmtGetFileId = null;
  /**
   * <code>stmtGetFingerprintTypeId</code> SQLDatabase -
   */
  private static PreparedStatement stmtGetFingerprintTypeId = null;
  /**
   * <code>stmtGetFingerprint</code> SQLDatabase -
   */
  private static PreparedStatement stmtGetFingerprint = null;
  /**
   * <code>stmtGetFingerprintId</code> SQLDatabase -
   */
  private static PreparedStatement stmtGetFingerprintId = null;
  /**
   * <code>stmtGetInfo</code> SQLDatabase -
   */
  private static PreparedStatement stmtGetInfo = null;
  /**
   * <code>stmtGetStatus</code> SQLDatabase -
   */
  private static PreparedStatement stmtGetStatus = null;
  /**
   * <code>stmtInsertFilename</code> SQLDatabase -
   */
  private static PreparedStatement stmtInsertFilename = null;
  /**
   * <code>stmtInsertFingerprint</code> SQLDatabase -
   */
  private static PreparedStatement stmtInsertFingerprint = null;
  /**
   * <code>stmtInsertInfo</code> SQLDatabase -
   */
  private static PreparedStatement stmtInsertInfo = null;
  /**
   * <code>stmtInsertInfoType</code> SQLDatabase -
   */
  private static PreparedStatement stmtInsertInfoType = null;
  /**
   * <code>stmtInsertSimilarity</code> SQLDatabase -
   */
  private static PreparedStatement stmtInsertSimilarity = null;
  /**
   * <code>stmtMoveFile</code> SQLDatabase -
   */
  private static PreparedStatement stmtMoveFile = null;
  /**
   * <code>stmtUpdateSimilarity</code> SQLDatabase -
   */
  private static PreparedStatement stmtUpdateSimilarity = null;
  /**
   * <code>stmtUpdateStatus</code> SQLDatabase -
   */
  private static PreparedStatement stmtUpdateStatus = null;
  /**
   * <code>TABLE</code> SQLDatabase - used to extract only the user tables from the database
   */
  private static final String TABLE = "TABLE"; //$NON-NLS-1$
  /**
   * <code>TBL_DUPLICATE</code> SQLDatabase -
   */
  private static final String TBL_DUPLICATE = "tblDUP"; //$NON-NLS-1$
  /**
   * <code>TBL_FILEINFO</code> SQLDatabase -
   */
  private static final String TBL_FILEINFO = "tblFI"; //$NON-NLS-1$
  /**
   * <code>TBL_FILEINFOTYPE</code> SQLDatabase -
   */
  private static final String TBL_FILEINFOTYPE = "tblFIT"; //$NON-NLS-1$
  /**
   * <code>TBL_FILENAME</code> SQLDatabase -
   */
  private static final String TBL_FILENAME = "tblFN"; //$NON-NLS-1$
  /**
   * <code>TBL_FINGERPRINT</code> SQLDatabase -
   */
  private static final String TBL_FINGERPRINT = "tblFP"; //$NON-NLS-1$
  /**
   * <code>TBL_FINGERPRINTTYPE</code> SQLDatabase -
   */
  private static final String TBL_FINGERPRINTTYPE = "tblFPT"; //$NON-NLS-1$
  /**
   * <code>TBL_STATUS</code> SQLDatabase -
   */
  private static final String TBL_STATUS = "tblSTS"; //$NON-NLS-1$
  /**
   * <code>url</code> SQLDatabase -
   */
  protected static String url = null;
  /**
   * <code>URL</code> SQLDatabase -
   */
  private static final String URL = "url"; //$NON-NLS-1$
  /**
   * <code>user</code> SQLDatabase -
   */
  protected static String user = null;
  /**
   * <code>USER</code> SQLDatabase -
   */
  private static final String USER = "user"; //$NON-NLS-1$
  /**
   * <code>keepalive</code> SQLDatabase - keep connection alive
   */
  private KeepAlive keepalive = null;
  static {
    Configure.load();
    // Load a database driver
    driver = Configure.getProperty(DRIVER, DEFAULT_JDBC_DRIVER, SQLDatabase.class.getName());
    url = Configure.getProperty(URL, DEFAULT_DATABASE, SQLDatabase.class.getName());
    user = Configure.getProperty(USER, DEFAULT_USER, SQLDatabase.class.getName());
    password = Configure.getProperty(PASSWORD, DEFAULT_PASSWORD, SQLDatabase.class.getName());
  }

  /**
   * This method replaced MySQL delimiters with characters that will work better.
   * 
   * @param pstrKey
   * @return String
   */
  private static String convertKey(final String pstrKey) {
    return pstrKey.replace(Chars.QUOTE, Chars.STAR).replace(Chars.DQUOTE, Chars.COLON);
  }

  /**
   * This method finds the fingerprint by id
   * 
   * @param pFile
   * @param pId
   * @return AbstractFingerprint
   */
  private static AbstractFingerprint findFingerprint(final FingerprintFile pFile, final long pId) {
    for (final AbstractFingerprint fp : pFile.getFingerprints().values()) {
      if (fp.getId() == pId) {
        return fp;
      }
    }
    return null;
  }

  /**
   * <code>sqlCommands</code> SQLDatabase -
   */
  protected SQLCommands sqlCommands = new SQLCommands();

  /**
   * SQLDatabase constructor.
   */
  SQLDatabase() {
    super();
  }

  /**
   * This method checks that the database is still alive and reconnects it if necessary.
   */
  synchronized void checkAlive() {
    if (conn == null) {
      // can not connect
      return;
    }
    ResultSet rs = null;
    try {
      if (stmtCheckAlive == null) {
        stmtCheckAlive = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 100));
      }
      rs = stmtCheckAlive.executeQuery();
    } catch (final SQLException se) {
      close();
      connect();
    } finally {
      try {
        if (rs != null) {
          rs.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
  }

  /**
   * This method checks that all the statuses are loaded
   */
  private void checkStatus() {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return;
    }
    if (log.isDebugEnabled()) {
      log.debug(Debug.getString(SQLDatabase.class, 9));
    }
    PreparedStatement statement = null;
    PreparedStatement insert = null;
    ResultSet rs = null;
    try {
      statement = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 103));
      insert = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 202));
      for (final Status s : Status.values()) {
        statement.setInt(1, s.getValue());
        rs = statement.executeQuery();
        if (!rs.next()) {
          insert.setInt(1, s.getValue());
          insert.setString(2, s.getDescription());
          insert.executeUpdate();
        }
        rs.close();
      }
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 10));
      }
    } catch (final SQLException se) {
      log.warn(Debug.getString(SQLDatabase.class, 11), se);
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
        if (statement != null) {
          statement.close();
        }
        if (insert != null) {
          insert.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
  }

  /**
   * This method checks whether the necessary tables have been created and creates them according to the properties file.
   */
  protected void checkTables() {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return;
    }
    try {
      List<String> tables = new ArrayList<String>();
      DatabaseMetaData dmd = conn.getMetaData();
      ResultSet rs1 = dmd.getSchemas();
      boolean schema = false;
      while (rs1.next()) {
        String ss = rs1.getString(1);
        ResultSet rs2 = dmd.getTables(null, ss, Strings.PERCENT, new String[]{
          TABLE
        });
        while (rs2.next()) {
          tables.add(rs2.getString(3).toUpperCase());
        }
        schema = true;
        rs2.close();
      }
      if (!schema) {
        // database does not use schemas - just get the tables
        ResultSet rs2 = dmd.getTables(null, null, Strings.PERCENT, new String[]{
          TABLE
        });
        while (rs2.next()) {
          tables.add(rs2.getString(3).toUpperCase());
        }
        rs2.close();
      }
      rs1.close();
      log.debug(tables.toString());
      if (!tables.contains(sqlCommands.getString(TBL_STATUS).toUpperCase())) {
        // create status and related objects
        for (int i = 0; i < 100; i += 10) {
          createDatabaseObject(i + 1);
        }
      }
      if (!tables.contains(sqlCommands.getString(TBL_FILEINFOTYPE).toUpperCase())) {
        // create fileinfotype and related objects
        for (int i = 0; i < 100; i += 10) {
          createDatabaseObject(i + 2);
        }
      }
      if (!tables.contains(sqlCommands.getString(TBL_FINGERPRINTTYPE).toUpperCase())) {
        // create fingerprinttype and related objects
        for (int i = 0; i < 100; i += 10) {
          createDatabaseObject(i + 3);
        }
      }
      if (!tables.contains(sqlCommands.getString(TBL_FILENAME).toUpperCase())) {
        // create filename and related objects
        for (int i = 0; i < 100; i += 10) {
          createDatabaseObject(i + 4);
        }
      }
      if (!tables.contains(sqlCommands.getString(TBL_FILEINFO).toUpperCase())) {
        // create fileinfo and related objects
        for (int i = 0; i < 100; i += 10) {
          createDatabaseObject(i + 5);
        }
      }
      if (!tables.contains(sqlCommands.getString(TBL_FINGERPRINT).toUpperCase())) {
        // create fingerprint table and related objects
        for (int i = 0; i < 100; i += 10) {
          createDatabaseObject(i + 6);
        }
      }
      if (!tables.contains(sqlCommands.getString(TBL_DUPLICATE).toUpperCase())) {
        // create duplicate table and related objects
        for (int i = 0; i < 100; i += 10) {
          createDatabaseObject(i + 7);
        }
      }
    } catch (final SQLException se) {
      log.warn(Debug.getString(SQLDatabase.class, 95), se);
    }
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#close()
   */
  public void close() {
    if (log.isDebugEnabled()) {
      log.debug(Debug.getString(SQLDatabase.class, 4));
    }
    try {
      keepalive.stop();
      if (stmtGetFingerprint != null) {
        stmtGetFingerprint.close();
        stmtGetFingerprint = null;
      }
      if (stmtInsertFilename != null) {
        stmtInsertFilename.close();
        stmtInsertFilename = null;
      }
      if (stmtUpdateStatus != null) {
        stmtUpdateStatus.close();
        stmtUpdateStatus = null;
      }
      if (stmtGetFileId != null) {
        stmtGetFileId.close();
        stmtGetFileId = null;
      }
      if (stmtGetFingerprintId != null) {
        stmtGetFingerprintId.close();
        stmtGetFingerprintId = null;
      }
      if (stmtGetInfo != null) {
        stmtGetInfo.close();
        stmtGetInfo = null;
      }
      if (stmtGetFingerprintTypeId != null) {
        stmtGetFingerprintTypeId.close();
        stmtGetFingerprintTypeId = null;
      }
      if (stmtInsertFingerprint != null) {
        stmtInsertFingerprint.close();
        stmtInsertFingerprint = null;
      }
      if (stmtGetStatus != null) {
        stmtGetStatus.close();
        stmtGetStatus = null;
      }
      if (stmtInsertInfo != null) {
        stmtInsertInfo.close();
        stmtInsertInfo = null;
      }
      if (stmtInsertInfoType != null) {
        stmtInsertInfoType.close();
        stmtInsertInfoType = null;
      }
      if (stmtDeleteInfo != null) {
        stmtDeleteInfo.close();
        stmtDeleteInfo = null;
      }
      if (stmtDeleteDuplicate != null) {
        stmtDeleteDuplicate.close();
        stmtDeleteDuplicate = null;
      }
      if (stmtDeleteFile != null) {
        stmtDeleteFile.close();
        stmtDeleteFile = null;
      }
      if (stmtMoveFile != null) {
        stmtMoveFile.close();
        stmtMoveFile = null;
      }
      if (stmtDeleteFingerprints != null) {
        stmtDeleteFingerprints.close();
        stmtDeleteFingerprints = null;
      }
      if (stmtCheckAlive != null) {
        stmtCheckAlive.close();
        stmtCheckAlive = null;
      }
      if (conn != null) {
        conn.close();
      }
    } catch (final SQLException se) {
      log.warn(Debug.getString(SQLDatabase.class, 5), se);
    } catch (final Exception t) {
      log.error(Debug.getString(SQLDatabase.class, 5), t);
    } finally {
      conn = null;
    }
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#connect()
   */
  public boolean connect() {
    try {
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 1));
      }
      // try to connect
      Class.forName(driver);
      if ((conn != null) && !conn.isClosed()) {
        try {
          conn.close();
        } catch (final SQLException se) {
          // ignore we are closing
        }
      }
      conn = DriverManager.getConnection(url, user, password);
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 2));
      }
      checkTables();
      checkStatus();
      loadFingerprintTypes();
      loadInfoTypes();
      keepalive = new KeepAlive();
      final Thread t = new Thread(keepalive, KeepAlive.class.getSimpleName());
      t.setDaemon(true);
      t.start();
      return true;
    } catch (final ClassNotFoundException cnf) {
      final List<String> replace = new ArrayList<String>();
      replace.add(cnf.getLocalizedMessage());
      log.error(Debug.getString(SQLDatabase.class, 3, replace));
      conn = null;
    } catch (final SQLException sqle) {
      final List<String> replace = new ArrayList<String>();
      replace.add(sqle.getLocalizedMessage());
      log.error(Debug.getString(SQLDatabase.class, 3, replace));
      conn = null;
    }
    return false;
  }

  /**
   * This method creates a table in the database.
   * 
   * @param id
   * @return boolean
   */
  private boolean createDatabaseObject(final int id) {
    final String command = sqlCommands.getString(SQLDatabase.class, id);
    if (command == null || command.length() <= 0
      || (command.charAt(0) == Chars.EXCLAMATION && command.charAt(command.length() - 1) == Chars.EXCLAMATION)) {
      return true;
    }
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(command);
      log.debug(Debug.getString(SQLDatabase.class, 96, replace));
    }
    Statement create = null;
    try {
      create = conn.createStatement();
      create.executeUpdate(command);
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 97));
      }
      return true;
    } catch (final SQLException se) {
      log.warn(Debug.getString(SQLDatabase.class, 98), se);
      return false;
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
        if (create != null) {
          create.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#delete(net.za.grasser.duplicate.file.FingerprintFile)
   */
  public synchronized void delete(final FingerprintFile pFf) {
    final String k1 = pFf.getKey();
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(k1);
      log.debug(Debug.getString(SQLDatabase.class, 67, replace));
    }
    final long id = pFf.getId();
    if (id < 0) {
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 80));
      }
      return;
    }
    // delete duplicates
    deleteDuplicates(id);
    // delete fingerprints
    deleteFingerprints(id);
    // delete info
    deleteInfo(id);
    // delete file
    deleteFile(id);
    if (log.isDebugEnabled()) {
      log.debug(Debug.getString(SQLDatabase.class, 68));
    }
  }

  /**
   * This method deletes the entries in the duplicate table.
   * 
   * @param fn
   */
  private void deleteDuplicates(final long fn) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return;
    }
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(String.valueOf(fn));
      log.debug(Debug.getString(SQLDatabase.class, 99, replace));
    }
    ResultSet rs = null;
    try {
      if (stmtGetFingerprintId == null) {
        stmtGetFingerprintId = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 108));
      }
      if (stmtDeleteDuplicate == null) {
        stmtDeleteDuplicate = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 403));
      }
      stmtGetFingerprintId.setLong(1, fn);
      rs = stmtGetFingerprintId.executeQuery();
      while (rs.next()) {
        stmtDeleteDuplicate.setLong(1, rs.getLong(1));
        stmtDeleteDuplicate.setLong(2, rs.getLong(1));
        stmtDeleteDuplicate.executeUpdate();
      }
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 100));
      }
    } catch (final SQLException se) {
      final List<String> replace = new ArrayList<String>();
      replace.add(String.valueOf(fn));
      log.warn(Debug.getString(SQLDatabase.class, 101, replace), se);
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
        if (rs != null) {
          rs.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
  }

  /**
   * @param fn
   */
  private void deleteFile(final long fn) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return;
    }
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(String.valueOf(fn));
      log.debug(Debug.getString(SQLDatabase.class, 77, replace));
    }
    try {
      if (stmtDeleteFile == null) {
        stmtDeleteFile = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 400));
      }
      stmtDeleteFile.setLong(1, fn);
      stmtDeleteFile.executeUpdate();
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 78));
      }
    } catch (final SQLException se) {
      final List<String> replace = new ArrayList<String>();
      replace.add(String.valueOf(fn));
      log.warn(Debug.getString(SQLDatabase.class, 79, replace), se);
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
      } catch (final SQLException se) {
        log.warn(Debug.getString(SQLDatabase.class, 33), se);
      }
    }
  }

  /**
   * @param fn
   */
  private void deleteFingerprints(final long fn) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return;
    }
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(String.valueOf(fn));
      log.debug(Debug.getString(SQLDatabase.class, 74, replace));
    }
    try {
      if (stmtDeleteFingerprints == null) {
        stmtDeleteFingerprints = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 402));
      }
      stmtDeleteFingerprints.setLong(1, fn);
      stmtDeleteFingerprints.executeUpdate();
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 75));
      }
    } catch (final SQLException se) {
      final List<String> replace = new ArrayList<String>();
      replace.add(String.valueOf(fn));
      log.warn(Debug.getString(SQLDatabase.class, 76, replace), se);
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
      } catch (final SQLException se) {
        log.warn(Debug.getString(SQLDatabase.class, 33), se);
      }
    }
  }

  /**
   * @param pFile
   */
  private void deleteInfo(final long pFile) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return;
    }
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(String.valueOf(pFile));
      log.debug(Debug.getString(SQLDatabase.class, 71, replace));
    }
    try {
      if (stmtDeleteInfo == null) {
        stmtDeleteInfo = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 401));
      }
      stmtDeleteInfo.setLong(1, pFile);
      stmtDeleteInfo.executeUpdate();
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 72));
      }
    } catch (final SQLException se) {
      final List<String> replace = new ArrayList<String>();
      replace.add(String.valueOf(pFile));
      log.warn(Debug.getString(SQLDatabase.class, 73, replace), se);
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
      } catch (final SQLException se) {
        log.warn(Debug.getString(SQLDatabase.class, 33), se);
      }
    }
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#get(net.za.grasser.duplicate.file.FingerprintFile)
   */
  public synchronized boolean get(final FingerprintFile pBf) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return false;
    }
    final String key = pBf.getKey();
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(key);
      log.debug(Debug.getString(SQLDatabase.class, 50, replace));
    }
    final IDStatus s = getFileNameId(key);
    if (s != null) {
      pBf.setId(s.fileid);
      pBf.setStatus(s.status);
      getFingerprints(s.fileid, pBf);
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 51));
      }
      return true;
    }
    if (log.isDebugEnabled()) {
      log.debug(Debug.getString(SQLDatabase.class, 52));
    }
    return false;
  }

  /**
   * gets the id for the filename
   * 
   * @param pKey filename
   * @return filename id
   */
  private IDStatus getFileNameId(final String pKey) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return null;
    }
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(pKey);
      log.debug(Debug.getString(SQLDatabase.class, 18, replace));
    }
    ResultSet rs = null;
    try {
      if (stmtGetFileId == null) {
        stmtGetFileId = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 104));
      }
      stmtGetFileId.setString(1, convertKey(pKey));
      rs = stmtGetFileId.executeQuery();
      if (rs.next()) {
        final IDStatus s = new IDStatus();
        s.fileid = rs.getInt(1);
        s.status = Status.find(rs.getInt(2));
        if (log.isDebugEnabled()) {
          final List<String> replace = new ArrayList<String>();
          replace.add(String.valueOf(s.fileid));
          replace.add(s.status.toString());
          log.debug(Debug.getString(SQLDatabase.class, 19, replace));
        }
        return s;
      }
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 20));
      }
      return null;
    } catch (final SQLException se) {
      final List<String> replace = new ArrayList<String>();
      replace.add(pKey);
      log.warn(Debug.getString(SQLDatabase.class, 21, replace), se);
      return null;
    } finally {
      try {
        if (rs != null) {
          rs.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
  }

  /**
   * gets the fingerprinttype id from the hash (loaded at startup), inserts it if not present
   * 
   * @param pFileType
   * @return fingerprinttype id
   */
  private Integer getFingerprintTypeId(final String pFileType) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return null;
    }
    Integer ft = fingerprintTypes.get(pFileType);
    if (ft == null) {
      if (log.isDebugEnabled()) {
        final List<String> replace = new ArrayList<String>();
        replace.add(pFileType);
        log.debug(Debug.getString(SQLDatabase.class, 41, replace));
      }
      ResultSet rs = null;
      try {
        if (stmtGetFingerprintTypeId == null) {
          stmtGetFingerprintTypeId = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 201), new String[]{
            sqlCommands.getString(COL_FINGERPRINTTYPEID)
          });
        }
        stmtGetFingerprintTypeId.setString(1, pFileType);
        stmtGetFingerprintTypeId.executeUpdate();
        rs = stmtGetFingerprintTypeId.getGeneratedKeys();
        if (rs.next()) {
          ft = new Integer(rs.getInt(1));
          fingerprintTypes.put(pFileType, ft);
        }
        if (log.isDebugEnabled()) {
          final List<Integer> replace = new ArrayList<Integer>();
          replace.add(ft);
          log.debug(Debug.getString(SQLDatabase.class, 42, replace));
        }
        return ft;
      } catch (final SQLException se) {
        final List<String> replace = new ArrayList<String>();
        replace.add(pFileType);
        log.warn(Debug.getString(SQLDatabase.class, 43, replace), se);
        return null;
      } finally {
        try {
          if (!conn.getAutoCommit()) {
            conn.commit();
          }
          if (rs != null) {
            rs.close();
          }
        } catch (final SQLException se) {
          log.warn(Debug.getString(SQLDatabase.class, 33), se);
        }
      }
    }
    return ft;
  }

  /**
   * This method will read the fingerprints from the database
   * 
   * @param piFileId
   * @param pBf
   */
  private void getFingerprints(final long piFileId, final FingerprintFile pBf) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return;
    }
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(String.valueOf(piFileId));
      log.debug(Debug.getString(SQLDatabase.class, 26, replace));
    }
    ResultSet rs = null;
    try {
      if (stmtGetFingerprint == null) {
        stmtGetFingerprint = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 105));
      }
      stmtGetFingerprint.setLong(1, piFileId);
      rs = stmtGetFingerprint.executeQuery();
      while (rs.next()) {
        final int id = rs.getInt(1);
        final String key = rs.getString(2);
        final byte[] fp = Base64.decode(rs.getString(3));
        final AbstractFingerprint af = pBf.getFingerprints().get(key);
        if (af != null) {
          af.setId(id);
          af.setFingerprint(fp);
        }
      }
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 27));
      }
    } catch (final SQLException se) {
      log.warn(Debug.getString(SQLDatabase.class, 28), se);
    } finally {
      try {
        if (rs != null) {
          rs.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#getInfo(net.za.grasser.duplicate.file.FingerprintFile)
   */
  public synchronized boolean getInfo(final FingerprintFile pFf) {
    final String key = pFf.getKey();
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(key);
      log.debug(Debug.getString(SQLDatabase.class, 61, replace));
    }
    final long id = pFf.getId();
    if (id < 0) {
      final IDStatus s = getFileNameId(key);
      if (s == null) {
        if (log.isDebugEnabled()) {
          log.debug(Debug.getString(SQLDatabase.class, 62));
        }
        return false;
      }
    }
    final Map<String, String> info = getInfo(id);
    if (info != null) {
      pFf.getInfo().putAll(info);
    }
    if (log.isDebugEnabled()) {
      log.debug(Debug.getString(SQLDatabase.class, 63));
    }
    return true;
  }

  /**
   * This method gets all the info from the db
   * 
   * @param piFileId
   * @return Map<String, String>
   */
  private Map<String, String> getInfo(final long piFileId) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return null;
    }
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(String.valueOf(piFileId));
      log.debug(Debug.getString(SQLDatabase.class, 22, replace));
    }
    ResultSet rs = null;
    try {
      if (stmtGetInfo == null) {
        stmtGetInfo = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 106));
      }
      stmtGetInfo.setLong(1, piFileId);
      rs = stmtGetInfo.executeQuery();
      final Map<String, String> info = new HashMap<String, String>();
      while (rs.next()) {
        info.put(rs.getString(1), rs.getString(2));
      }
      if (info.isEmpty()) {
        if (log.isDebugEnabled()) {
          log.debug(Debug.getString(SQLDatabase.class, 23));
        }
        return null;
      }
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 24));
      }
      return info;
    } catch (final SQLException se) {
      log.warn(Debug.getString(SQLDatabase.class, 25), se);
    } finally {
      try {
        if (rs != null) {
          rs.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
    return null;
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#getSimilarity(net.za.grasser.duplicate.file.FingerprintFile, net.za.grasser.duplicate.file.FingerprintFile)
   */
  public List<CompareStatus> getSimilarity(final FingerprintFile pFf1, final FingerprintFile pFf2) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return null;
    }
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(pFf1.getPath());
      replace.add(pFf2.getPath());
      log.debug(Debug.getString(SQLDatabase.class, 89, replace));
    }
    ResultSet rs = null;
    try {
      if (stmtFindSimilarity == null) {
        stmtFindSimilarity = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 107));
      }
      final long k1 = pFf1.getId();
      final long k2 = pFf2.getId();
      final List<CompareStatus> sims = new ArrayList<CompareStatus>();
      if ((k1 < 0L) || (k2 < 0L)) {
        if (log.isDebugEnabled()) {
          log.debug(Debug.getString(SQLDatabase.class, 91));
        }
        for (final String key : pFf1.getFingerprints().keySet()) {
          final AbstractFingerprint afToCompare = pFf1.getFingerprints().get(key);
          final AbstractFingerprint afCompareTo = pFf2.getFingerprints().get(key);
          if ((afToCompare != null) && (afCompareTo != null)) {
            sims.add(new CompareStatus(afToCompare, afCompareTo));
          }
        }
        return sims;
      }
      stmtFindSimilarity.setLong(1, k1);
      stmtFindSimilarity.setLong(2, k2);
      rs = stmtFindSimilarity.executeQuery();
      while (rs.next()) {
        final long id1 = rs.getLong(1);
        final long id2 = rs.getLong(2);
        AbstractFingerprint fp1 = findFingerprint(pFf1, id1);
        if (fp1 == null) {
          // fingerprint not found in file 1, check file 2
          fp1 = findFingerprint(pFf2, id1);
        }
        AbstractFingerprint fp2 = findFingerprint(pFf1, id2);
        if (fp2 == null) {
          // fingerprint not found in file 1, check file 2
          fp2 = findFingerprint(pFf2, id2);
        }
        final CompareStatus cs = new CompareStatus(fp1, fp2);
        cs.setSimilarity(rs.getFloat(3));
        sims.add(cs);
      }
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 92));
      }
      return sims;
    } catch (final SQLException se) {
      // problem
      log.warn(Debug.getString(SQLDatabase.class, 93), se);
    } finally {
      try {
        if (rs != null) {
          rs.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
    if (log.isDebugEnabled()) {
      log.debug(Debug.getString(SQLDatabase.class, 90));
    }
    return null;
  }

  /**
   * This method inserts the file into the database.
   * 
   * @param pKey
   * @param pStatus
   * @return id of the inserted file
   */
  private long insertFileName(final String pKey, final int pStatus) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return -1;
    }
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(pKey);
      replace.add(String.valueOf(pStatus));
      log.debug(Debug.getString(SQLDatabase.class, 30, replace));
    }
    ResultSet rs = null;
    try {
      if (stmtInsertFilename == null) {
        stmtInsertFilename = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 203), new String[]{
          sqlCommands.getString(COL_FILENAMEID)
        });
      }
      stmtInsertFilename.setString(1, convertKey(pKey));
      stmtInsertFilename.setInt(2, pStatus);
      stmtInsertFilename.executeUpdate();
      rs = stmtInsertFilename.getGeneratedKeys();
      long id = -1;
      if (rs.next()) {
        id = rs.getInt(1);
        if (log.isDebugEnabled()) {
          final List<String> replace = new ArrayList<String>();
          replace.add(String.valueOf(id));
          log.debug(Debug.getString(SQLDatabase.class, 31, replace));
        }
        return id;
      }
      if (log.isDebugEnabled()) {
        final List<String> replace = new ArrayList<String>();
        replace.add(String.valueOf(id));
        log.debug(Debug.getString(SQLDatabase.class, 31, replace));
      }
      return id;
    } catch (final SQLException se) {
      final IDStatus id = getFileNameId(pKey);
      if (id == null) {
        final List<String> replace = new ArrayList<String>();
        replace.add(pKey);
        log.warn(Debug.getString(SQLDatabase.class, 32, replace), se);
        return -1;
      }
      if (log.isDebugEnabled()) {
        final List<String> replace = new ArrayList<String>();
        replace.add(String.valueOf(id.fileid));
        log.debug(Debug.getString(SQLDatabase.class, 31, replace));
      }
      return id.fileid;
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
        if (rs != null) {
          rs.close();
        }
      } catch (final SQLException se) {
        log.warn(Debug.getString(SQLDatabase.class, 33), se);
      }
    }
  }

  /**
   * @param fn
   * @param key
   * @param pBf
   */
  private void insertFingerprint(final long fn, final String key, final AbstractFingerprint pBf) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return;
    }
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(String.valueOf(fn));
      replace.add(key);
      replace.add(HexArray.makeString(pBf.getFingerprint()));
      log.debug(Debug.getString(SQLDatabase.class, 44, replace));
    }
    final byte[] fingerprint = pBf.getFingerprint();
    ResultSet rs = null;
    try {
      final Integer ft = getFingerprintTypeId(key);
      if (fingerprint == null) {
        return;
      }
      if (stmtInsertFingerprint == null) {
        stmtInsertFingerprint = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 204), new String[]{
          sqlCommands.getString(COL_FINGERPRINTID)
        });
      }
      stmtInsertFingerprint.setLong(1, fn);
      stmtInsertFingerprint.setInt(2, ft.intValue());
      stmtInsertFingerprint.setString(3, Base64.encodeBytes(fingerprint));
      stmtInsertFingerprint.executeUpdate();
      rs = stmtInsertFingerprint.getGeneratedKeys();
      long id = -1;
      if (rs.next()) {
        id = rs.getInt(1);
      }
      pBf.setId(id);
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 45));
      }
    } catch (final SQLException se) {
      // fingerprint already exists.
      log.warn(Debug.getString(SQLDatabase.class, 46), se);
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
        if (rs != null) {
          rs.close();
        }
      } catch (final SQLException se) {
        log.warn(Debug.getString(SQLDatabase.class, 33), se);
      }
    }
  }

  /**
   * @param pFile
   * @param pInfo
   */
  private void insertInfo(final long pFile, final Map<String, String> pInfo) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return;
    }
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(String.valueOf(pFile));
      log.debug(Debug.getString(SQLDatabase.class, 37, replace));
    }
    if (pInfo == null) {
      return;
    }
    try {
      if (stmtInsertInfo == null) {
        stmtInsertInfo = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 205));
      }
      for (final String str : pInfo.keySet()) {
        Integer id = infoTypes.get(str);
        if (id == null) {
          // insert info type first
          id = insertInfoType(str);
          if (id == null) {
            // tables probably don't exist
            return;
          }
        }
        stmtInsertInfo.setInt(1, id.intValue());
        stmtInsertInfo.setLong(2, pFile);
        stmtInsertInfo.setString(3, pInfo.get(str));
        try {
          stmtInsertInfo.executeUpdate();
        } catch (final SQLException se) {
          // just this info was not inserted.
          if (log.isTraceEnabled()) {
            final List<String> replace = new ArrayList<String>();
            replace.add(str);
            replace.add(pInfo.get(str));
            log.trace(Debug.getString(SQLDatabase.class, 38, replace));
          }
        }
      }
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 39));
      }
    } catch (final SQLException se) {
      final List<String> replace = new ArrayList<String>();
      replace.add(String.valueOf(pFile));
      log.warn(Debug.getString(SQLDatabase.class, 40, replace), se);
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
      } catch (final SQLException se) {
        log.warn(Debug.getString(SQLDatabase.class, 33), se);
      }
    }
  }

  /**
   * This method inserts new info types into the info type table.
   * 
   * @param pType
   * @return Integer
   */
  private Integer insertInfoType(final String pType) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return null;
    }
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(pType);
      log.debug(Debug.getString(SQLDatabase.class, 34, replace));
    }
    ResultSet rs = null;
    try {
      if (stmtInsertInfoType == null) {
        stmtInsertInfoType = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 200), new String[]{
          sqlCommands.getString(COL_FILEINFOTYPEID)
        });
      }
      stmtInsertInfoType.setString(1, pType);
      stmtInsertInfoType.executeUpdate();
      rs = stmtInsertInfoType.getGeneratedKeys();
      Integer i = null;
      if (rs.next()) {
        i = new Integer(rs.getInt(1));
        infoTypes.put(pType, i);
      }
      if (log.isDebugEnabled()) {
        final List<String> replace = new ArrayList<String>();
        replace.add(String.valueOf(i));
        log.debug(Debug.getString(SQLDatabase.class, 35, replace));
      }
      return i;
    } catch (final SQLException se) {
      final List<String> replace = new ArrayList<String>();
      replace.add(pType);
      log.warn(Debug.getString(SQLDatabase.class, 36, replace), se);
      return null;
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
        if (rs != null) {
          rs.close();
        }
      } catch (final SQLException se) {
        log.warn(Debug.getString(SQLDatabase.class, 33), se);
      }
    }
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#isConnected()
   */
  public boolean isConnected() {
    try {
      return (conn != null) && !conn.isClosed();
    } catch (SQLException e) {
      return false;
    }
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#isNotDuplicate(net.za.grasser.duplicate.file.FingerprintFile, net.za.grasser.duplicate.file.FingerprintFile)
   */
  public synchronized boolean isNotDuplicate(final FingerprintFile pFf1, final FingerprintFile pFf2) {
    List<CompareStatus> sims = getSimilarity(pFf1, pFf2);
    if (sims != null) {
      for (CompareStatus stat : sims) {
        if (stat.getSimilarity() != CompareStatus.COMPARED_NOT_DUPLICATE) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * This method loads a list of fingerprint types. Quicker than looking it up in the DB all the time.
   */
  private void loadFingerprintTypes() {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return;
    }
    if (log.isDebugEnabled()) {
      log.debug(Debug.getString(SQLDatabase.class, 12));
    }
    PreparedStatement statement = null;
    ResultSet rs = null;
    fingerprintTypes = new HashMap<String, Integer>();
    try {
      statement = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 102));
      rs = statement.executeQuery();
      while (rs.next()) {
        fingerprintTypes.put(rs.getString(2), new Integer(rs.getInt(1)));
      }
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 13));
      }
    } catch (final SQLException se) {
      log.warn(Debug.getString(SQLDatabase.class, 14), se);
    } finally {
      try {
        if (rs != null) {
          rs.close();
        }
        if (statement != null) {
          statement.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
  }

  /**
   * This method loads all info strings.
   */
  private void loadInfoTypes() {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return;
    }
    if (log.isDebugEnabled()) {
      log.debug(Debug.getString(SQLDatabase.class, 6));
    }
    PreparedStatement statement = null;
    ResultSet rs = null;
    infoTypes = new HashMap<String, Integer>();
    try {
      statement = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 101));
      rs = statement.executeQuery();
      while (rs.next()) {
        infoTypes.put(rs.getString(2), new Integer(rs.getInt(1)));
      }
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 7));
      }
    } catch (final SQLException se) {
      log.warn(Debug.getString(SQLDatabase.class, 8), se);
    } finally {
      try {
        if (rs != null) {
          rs.close();
        }
        if (statement != null) {
          statement.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#move(net.za.grasser.duplicate.file.FingerprintFile, java.io.File)
   */
  public synchronized void move(final FingerprintFile pFf, final File pPath) {
    final String k1 = pFf.getKey();
    final String p1 = pPath.getPath();
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(k1);
      replace.add(p1);
      log.debug(Debug.getString(SQLDatabase.class, 69, replace));
    }
    final long id = pFf.getId();
    if (id < 0) {
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 81));
      }
      return;
    }
    pFf.setParent(new DirectoryInfo(pPath.getParent()));
    pFf.setName(pPath.getName());
    // move file
    moveFile(id, pFf.getKey());
    if (log.isDebugEnabled()) {
      log.debug(Debug.getString(SQLDatabase.class, 70));
    }
  }

  /**
   * @param fn
   * @param key
   */
  private void moveFile(final long fn, final String key) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return;
    }
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(String.valueOf(fn));
      replace.add(key);
      log.debug(Debug.getString(SQLDatabase.class, 82, replace));
    }
    try {
      if (stmtMoveFile == null) {
        stmtMoveFile = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 300));
      }
      stmtMoveFile.setLong(1, fn);
      stmtMoveFile.setString(2, convertKey(key));
      stmtMoveFile.executeUpdate();
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 83));
      }
    } catch (final SQLException se) {
      final List<String> replace = new ArrayList<String>();
      replace.add(String.valueOf(fn));
      replace.add(key);
      log.warn(Debug.getString(SQLDatabase.class, 84, replace), se);
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
      } catch (final SQLException se) {
        log.warn(Debug.getString(SQLDatabase.class, 33), se);
      }
    }
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#put(net.za.grasser.duplicate.file.FingerprintFile)
   */
  public synchronized void put(final FingerprintFile pBf) {
    final String key = pBf.getKey();
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(key);
      log.debug(Debug.getString(SQLDatabase.class, 47, replace));
    }
    final long fn = insertFileName(key, pBf.getStatus().getValue());
    if (fn == -1L) {
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 48));
      }
      return;
    }
    pBf.setId(fn);
    for (final Map.Entry<String, AbstractFingerprint> af : pBf.getFingerprints().entrySet()) {
      insertFingerprint(fn, af.getKey(), af.getValue());
    }
    if (log.isDebugEnabled()) {
      log.debug(Debug.getString(SQLDatabase.class, 49));
    }
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#putInfo(net.za.grasser.duplicate.file.FingerprintFile)
   */
  public synchronized void putInfo(final FingerprintFile pFf) {
    final String key = pFf.getKey();
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(key);
      log.debug(Debug.getString(SQLDatabase.class, 64, replace));
    }
    final long id = pFf.getId();
    if (id < 0) {
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 65));
      }
      return;
    }
    if (pFf.hasInfo()) {
      insertInfo(id, pFf.getInfo());
    }
    if (log.isDebugEnabled()) {
      log.debug(Debug.getString(SQLDatabase.class, 66));
    }
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#putSimilarity(java.util.List)
   */
  public void putSimilarity(final List<CompareStatus> pStatus) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return;
    }
    if (pStatus == null) {
      return;
    }
    if (log.isDebugEnabled() && !pStatus.isEmpty()) {
      final FingerprintFile ff1 = pStatus.get(0).getFingerprint1().getFile();
      final FingerprintFile ff2 = pStatus.get(0).getFingerprint2().getFile();
      final List<String> replace = new ArrayList<String>();
      replace.add(ff1.getPath());
      replace.add(ff2.getPath());
      double max = CompareStatus.NOT_COMPARED;
      for (final CompareStatus s : pStatus) {
        if (s.getSimilarity() > max) {
          max = s.getSimilarity();
        }
      }
      replace.add(String.valueOf(max));
      log.debug(Debug.getString(SQLDatabase.class, 85, replace));
    }
    try {
      if (stmtInsertSimilarity == null) {
        stmtInsertSimilarity = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 206));
      }
      for (final CompareStatus s : pStatus) {
        if (s.getSimilarity() >= CompareStatus.COMPARED_NOT_DUPLICATE) {
          try {
            final AbstractFingerprint af1 = s.getFingerprint1();
            final AbstractFingerprint af2 = s.getFingerprint2();
            if (af1.getId() < 0L) {
              put(af1.getFile());
            }
            if (af2.getId() < 0L) {
              put(af2.getFile());
            }
            if ((af1.getId() > 0L) && (af2.getId() > 0L)) {
              stmtInsertSimilarity.setLong(1, af1.getId());
              stmtInsertSimilarity.setLong(2, af2.getId());
              stmtInsertSimilarity.setDouble(3, s.getSimilarity());
              stmtInsertSimilarity.executeUpdate();
            }
          } catch (final SQLException se) {
            // already exists but carry on
            log.warn(Debug.getString(SQLDatabase.class, 87), se);
          }
        }
      }
      if (log.isDebugEnabled()) {
        log.debug(Debug.getString(SQLDatabase.class, 86));
      }
    } catch (final SQLException se) {
      // could not create statement
      log.warn(Debug.getString(SQLDatabase.class, 88), se);
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
      } catch (final SQLException se) {
        log.warn(Debug.getString(SQLDatabase.class, 33), se);
      }
    }
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#setNotDuplicate(net.za.grasser.duplicate.file.FingerprintFile,
   *      net.za.grasser.duplicate.file.FingerprintFile)
   */
  public synchronized void setNotDuplicate(final FingerprintFile pFf1, final FingerprintFile pFf2) {
    if (log.isDebugEnabled()) {
      final List<String> replace = new ArrayList<String>();
      replace.add(pFf1.getPath());
      replace.add(pFf2.getPath());
      log.debug(Debug.getString(SQLDatabase.class, 57, replace));
    }
    List<CompareStatus> clist = new ArrayList<CompareStatus>();
    for (final String key : pFf1.getFingerprints().keySet()) {
      if (pFf2.getFingerprints().containsKey(key)) {
        AbstractFingerprint af1 = pFf1.getFingerprints().get(key);
        AbstractFingerprint af2 = pFf2.getFingerprints().get(key);
        CompareStatus comp = new CompareStatus(af1, af2);
        comp.setSimilarity(CompareStatus.COMPARED_NOT_DUPLICATE);
        clist.add(comp);
      }
    }
    updateSimilarity(clist);
  }

  /**
   * This method updates the similarity of two files
   * 
   * @param pStatus
   */
  private void updateSimilarity(List<CompareStatus> pStatus) {
    if (conn == null) {
      log.warn(Debug.getString(SQLDatabase.class, 94));
      return;
    }
    try {
      if (stmtUpdateSimilarity == null) {
        stmtUpdateSimilarity = conn.prepareStatement(sqlCommands.getString(SQLDatabase.class, 301));
      }
      for (final CompareStatus s : pStatus) {
        if (s.getSimilarity() >= CompareStatus.COMPARED_NOT_DUPLICATE) {
          try {
            final AbstractFingerprint af1 = s.getFingerprint1();
            final AbstractFingerprint af2 = s.getFingerprint2();
            if (af1.getId() < 0L) {
              put(af1.getFile());
            }
            if (af2.getId() < 0L) {
              put(af2.getFile());
            }
            if ((af1.getId() > 0L) || (af2.getId() > 0L)) {
              stmtUpdateSimilarity.setDouble(1, s.getSimilarity());
              stmtUpdateSimilarity.setLong(2, af1.getId());
              stmtUpdateSimilarity.setLong(3, af2.getId());
              stmtUpdateSimilarity.setLong(4, af2.getId());
              stmtUpdateSimilarity.setLong(5, af1.getId());
              stmtUpdateSimilarity.executeUpdate();
              if (log.isDebugEnabled()) {
                log.debug(Debug.getString(SQLDatabase.class, 86));
              }
            }
          } catch (final SQLException se) {
            // already exists but carry on
            log.warn(Debug.getString(SQLDatabase.class, 87), se);
          }
        }
      }
    } catch (final SQLException se) {
      // could not create statement
      log.warn(Debug.getString(SQLDatabase.class, 88), se);
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
      } catch (final SQLException se) {
        log.warn(Debug.getString(SQLDatabase.class, 33), se);
      }
    }
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#getName()
   */
  public String getName() {
    if (conn == null) {
      return Strings.EMPTY;
    }
    try {
      return conn.getMetaData().getDatabaseProductName();
    } catch (SQLException e) {
      return Strings.EMPTY;
    }
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#getVersion()
   */
  public String getVersion() {
    if (conn == null) {
      return Strings.EMPTY;
    }
    try {
      return conn.getMetaData().getDatabaseProductVersion();
    } catch (SQLException e) {
      return Strings.EMPTY;
    }
  }
}
