/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.util.res;

import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import net.za.grasser.duplicate.Configure;
import net.za.grasser.duplicate.util.Files;

/**
 * This class handles the localised message strings.
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.3 $
 */
public class Messages {
  /**
   * <code>COUNTRY</code> Messages -
   */
  private static final String COUNTRY = "country"; //$NON-NLS-1$
  /**
   * <code>LANGUAGE</code> Messages -
   */
  private static final String LANGUAGE = "language"; //$NON-NLS-1$
  /**
   * <code>RESOURCE_BUNDLE</code> Messages -
   */
  private static final ResourceBundle RESOURCE_BUNDLE;
  static {
    Configure.load();
    Locale loc = Locale.getDefault();
    final String lang = Configure.getProperty(LANGUAGE, loc.getLanguage(), Messages.class.getName());
    final String cntry = Configure.getProperty(COUNTRY, loc.getCountry(), Messages.class.getName());
    RESOURCE_BUNDLE = ResourceBundle.getBundle(Files.BUNDLE_NAME, new Locale(lang, cntry));
  }

  /**
   * Constructor
   */
  private Messages() {
    super();
  }

  /**
   * This method returns a string for a given class name and id.
   * 
   * @param c
   * @param id
   * @return localaised String
   */
  public static String getString(final Class< ? > c, final int id) {
    return Resource.getString(RESOURCE_BUNDLE, c, id);
  }

  /**
   * This method returns a string for a given key. Replaces all {n) with the corresponding value from the replace list if n is an integer otherwise it will try
   * to resolve the name in {} from the <code>ResourceBundle</code>. Be careful, if the replacement string contains a reference to itself (or a circular
   * reference is created) an infinite loop will result.<BR>
   * Eg. DO NOT DO THIS:<BR>
   * s1={s2}<br>
   * s2={s1}<br>
   * <br>
   * This also cause problems:<br>
   * s1=some {s2} string<BR>
   * s2=cool {s2}<br>
   * 
   * @param c
   * @param id
   * @param replace
   * @return String
   */
  public static String getString(final Class< ? > c, final int id, final List< ? extends Object> replace) {
    return Resource.getString(RESOURCE_BUNDLE, c, id, replace);
  }

  /**
   * This method returns a string for a given class name and id string.
   * 
   * @param c
   * @param id
   * @return localaised String
   */
  public static String getString(final Class< ? > c, final String id) {
    return Resource.getString(RESOURCE_BUNDLE, c, id);
  }
}
