/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.util.res;

import java.util.ArrayList;
import java.util.List;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

import net.za.grasser.duplicate.util.Chars;
import net.za.grasser.duplicate.util.Strings;

import org.apache.log4j.Logger;

/**
 * This class handles the localised message strings.
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.6 $
 */
public class Resource {
  /**
   * <code>MESSAGE</code> Messages -
   */
  private static final String MESSAGE = "MESSAGE_"; //$NON-NLS-1$
  /**
   * <code>log</code> Messages -
   */
  private static final Logger log = Logger.getLogger(Resource.class);

  /**
   * This method breaks a string into a list by finding groups between leftSep and rightSep. Sparators can be nested.
   * 
   * @param input
   * @param leftSep
   * @param rightSep
   * @return List<string>
   */
  private static List<String> breakString(final String input, final char leftSep, final char rightSep) {
    final List<String> ret = new ArrayList<String>();
    if ((input != null) && !input.equals(Strings.EMPTY)) {
      final StringTokenizer st = new StringTokenizer(input, String.valueOf(leftSep) + String.valueOf(rightSep), true);
      int count = 0;
      final StringBuffer temp = new StringBuffer();
      while (st.hasMoreTokens()) {
        final String next = st.nextToken();
        if (next.equals(String.valueOf(leftSep))) {
          count++;
          temp.append(leftSep);
        } else if (next.equals(String.valueOf(rightSep))) {
          count--;
          temp.append(rightSep);
          if (count <= 0) {
            ret.add(temp.toString());
            temp.setLength(0);
            count = 0;
          }
        } else if (count <= 0) {
          ret.add(next);
        } else {
          temp.append(next);
        }
      }
      if (temp.length() > 0) {
        ret.add(temp.toString());
      }
    }
    return ret;
  }

  /**
   * This method evaluates an expression.
   * 
   * @param pExpr
   * @return boolean
   */
  @SuppressWarnings("unchecked")//$NON-NLS-1$
  private static boolean evaluate(final String pExpr) {
    int o = -1;
    String op = Strings.BEQ;
    if (pExpr.indexOf(Strings.GE) > 0) {
      o = pExpr.indexOf(Strings.GE);
      op = Strings.GE;
    } else if (pExpr.indexOf(Strings.LE) > 0) {
      o = pExpr.indexOf(Strings.LE);
      op = Strings.LE;
    } else if (pExpr.indexOf(Strings.BEQ) > 0) {
      o = pExpr.indexOf(Strings.BEQ);
      op = Strings.BEQ;
    } else if (pExpr.indexOf(Strings.NE) > 0) {
      o = pExpr.indexOf(Strings.NE);
      op = Strings.NE;
    } else if (pExpr.indexOf(Chars.LT) > 0) {
      o = pExpr.indexOf(Chars.LT);
      op = Strings.LT;
    } else if (pExpr.indexOf(Chars.GT) > 0) {
      o = pExpr.indexOf(Chars.GT);
      op = Strings.GT;
    } else if (pExpr.indexOf(Chars.EQ) > 0) {
      o = pExpr.indexOf(Chars.EQ);
      op = Strings.EQ;
    } else {
      o = pExpr.length();
      op = Strings.EMPTY;
    }
    if (o >= 0) {
      if (op.equals(Strings.EMPTY)) {
        return pExpr.equalsIgnoreCase(Strings.TRUE);
      }
      Comparable lhs;
      Comparable rhs;
      try {
        // first try it as an integer
        lhs = new Integer(pExpr.substring(0, o));
        rhs = new Integer(pExpr.substring(o + op.length()));
      } catch (final NumberFormatException nfe) {
        try {
          // then as double
          lhs = new Double(pExpr.substring(0, o));
          rhs = new Double(pExpr.substring(o + op.length()));
        } catch (final NumberFormatException n1) {
          // then string
          lhs = pExpr.substring(0, o);
          rhs = pExpr.substring(o + op.length());
        }
      }
      if (op.equals(Strings.GE)) {
        return lhs.compareTo(rhs) >= 0;
      } else if (op.equals(Strings.LE)) {
        return lhs.compareTo(rhs) <= 0;
      } else if (op.equals(Strings.LT)) {
        return lhs.compareTo(rhs) < 0;
      } else if (op.equals(Strings.GT)) {
        return lhs.compareTo(rhs) > 0;
      } else if (op.equals(Strings.BEQ) || op.equals(Strings.EQ)) {
        return lhs.compareTo(rhs) == 0;
      } else if (op.equals(Strings.NE)) {
        return lhs.compareTo(rhs) != 0;
      }
    }
    return false;
  }

  /**
   * This method returns a string for a given class name and id.
   * 
   * @param pBundle
   * @param c
   * @param id
   * @return localaised String
   */
  public static String getString(final ResourceBundle pBundle, final Class< ? > c, final int id) {
    return getString(pBundle, c, String.valueOf(id));
  }

  /**
   * This method returns a string for a given key. Replaces all {n) with the corresponding value from the replace list if n is an integer otherwise it will try
   * to resolve the name in {} from the <code>ResourceBundle</code>. Be careful, if the replacement string contains a reference to itself (or a circular
   * reference is created) an infinite loop will result.<BR>
   * Eg. DO NOT DO THIS:<BR>
   * s1={s2}<br>
   * s2={s1}<br>
   * <br>
   * This also cause problems:<br>
   * s1=some {s2} string<BR>
   * s2=cool {s2}<br>
   * 
   * @param pBundle
   * @param c
   * @param id
   * @param replace
   * @return String
   */
  public static String getString(final ResourceBundle pBundle, final Class< ? > c, final int id, final List< ? extends Object> replace) {
    final String s = getString(pBundle, c, String.valueOf(id));
    return replace(pBundle, s, replace);
  }

  /**
   * This method will try to extract a default message string from the class. On failure it will return the key wrapped in exclamation marks.
   * 
   * @param c
   * @param id
   * @return String
   */
  private static String getMessage(final Class< ? > c, final String id) {
    try {
      // this allows the class to set static strings for default messages
      return (String)c.getField(MESSAGE + id).get(null);
    } catch (final Exception t) {
      return Strings.EXCLAMATION + c.getSimpleName() + Strings.DOT + id + Strings.EXCLAMATION;
    }
  }

  /**
   * This method returns a string for a given class name and id string.
   * 
   * @param pBundle
   * @param c
   * @param id
   * @return localaised String
   */
  public static String getString(final ResourceBundle pBundle, final Class< ? > c, final String id) {
    if (pBundle == null) {
      return getMessage(c, id);
    }
    try {
      return getString(pBundle, c.getSimpleName() + Strings.DOT + id);
    } catch (final MissingResourceException e) {
      return getMessage(c, id);
    }
  }

  /**
   * This method returns a string for a given key. If pBundle is null, return the key wrapped in exclamation marks.
   * 
   * @param pBundle
   * @param key
   * @return String
   * @throws MissingResourceException
   */
  static String getString(final ResourceBundle pBundle, final String key) throws MissingResourceException {
    if (pBundle == null) {
      return Strings.EXCLAMATION + key + Strings.EXCLAMATION;
    }
    return pBundle.getString(key);
  }

  /**
   * This method does the actual replacement of {} expressions.
   * 
   * @param pBundle
   * @param pVal
   * @param replace
   * @return String
   */
  private static String replace(final ResourceBundle pBundle, final String pVal, final List< ? extends Object> replace) {
    try {
      final StringBuffer val = new StringBuffer();
      for (String s : breakString(pVal, Chars.OPN_BRACE, Chars.CLS_BRACE)) {
        if (s.indexOf(Chars.OPN_BRACE) >= 0) {
          s = replaceGroup(pBundle, s, replace);
        }
        val.append(s);
      }
      return val.toString();
    } catch (final Exception t) {
      log.error(pVal, t);
      return Strings.EXCLAMATION + pVal + Strings.EXCLAMATION;
    }
  }

  /**
   * This method replaces a group in the format {expr} by first checking for an if expression, then checking whther the expr is an int and then ceching the
   * resource bundle for a string.
   * 
   * @param pBundle
   * @param pInput
   * @param pReplace
   * @return String
   */
  private static String replaceGroup(final ResourceBundle pBundle, final String pInput, final List< ? extends Object> pReplace) {
    String in = pInput.substring(1);
    if (in.charAt(in.length() - 1) == Chars.CLS_BRACE) {
      in = in.substring(0, in.length() - 1);
    }
    if ((in.indexOf(Chars.QMARK) > 0) && (in.indexOf(Chars.COLON, in.indexOf(Chars.QMARK)) > 0)) {
      in = replaceIf(pBundle, in, pReplace);
    } else if ((in.indexOf(Chars.OPN_BRACE) >= 0) && (in.indexOf(Chars.CLS_BRACE) >= 0)) {
      final StringBuffer sb = new StringBuffer();
      for (String s : breakString(in, Chars.OPN_BRACE, Chars.CLS_BRACE)) {
        if (s.indexOf(Chars.OPN_BRACE) >= 0) {
          s = replace(pBundle, s, pReplace);
        }
        sb.append(s);
      }
      in = sb.toString();
    } else {
      try {
        final Object r = pReplace.get(Integer.parseInt(in));
        if (r == null) {
          in = Strings.EXCLAMATION + Strings.NULL + Strings.EXCLAMATION;
        } else {
          in = r.toString();
        }
      } catch (final Exception nfe) {
        try {
          in = getString(pBundle, in);
        } catch (final MissingResourceException e) {
          // leave strings with separators as is.
          if ((in.indexOf(Chars.OPN_BRACE) < 0) && (in.indexOf(Chars.CLS_BRACE) < 0) && (in.indexOf(Chars.QMARK) < 0)
            && (in.indexOf(Chars.COLON) < 0)) {
            in = Strings.EXCLAMATION + in + Strings.EXCLAMATION;
          }
        }
      }
    }
    return in;
  }

  /**
   * This method replaces an expression of the form expr?if_part:else_part by evaluating expr and choosing the correct part.
   * 
   * @param pBundle
   * @param input
   * @param replace
   * @return String
   */
  private static String replaceIf(final ResourceBundle pBundle, final String input, final List< ? extends Object> replace) {
    final List<String> l = breakString(input, Chars.QMARK, Chars.COLON);
    final String expr = replace(pBundle, l.get(0), replace);
    if (expr.charAt(0) != Chars.QMARK) {
      if (evaluate(expr)) {
        String s = l.get(1);
        if ((s.length() > 0) && (s.charAt(0) == Chars.QMARK)) {
          s = s.substring(1);
        }
        if ((s.length() > 0) && (s.charAt(s.length() - 1) == Chars.COLON)) {
          s = s.substring(0, s.length() - 1);
        }
        return replace(pBundle, s, replace);
      } else if (l.size() > 2) {
        final StringBuffer sb = new StringBuffer();
        for (int j = 2; j < l.size(); j++) {
          sb.append(l.get(j));
        }
        return replace(pBundle, sb.toString(), replace);
      } else {
        return Strings.EMPTY;
      }
    }
    return Strings.EXCLAMATION + input + Strings.EXCLAMATION;
  }

  /**
   * Constructor
   */
  private Resource() {
    super();
  }
}
