/**
 * DuMP3 version 0.3.2 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.util.res;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import net.za.grasser.duplicate.Configure;
import net.za.grasser.duplicate.util.Files;
import net.za.grasser.duplicate.util.Strings;

/**
 * This class ...
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.6 $
 */
public class SQLCommands {
  /**
   * <code>DEFAULT_DIALECT</code> SQLCommands - mysql
   */
  private static final String DEFAULT_DIALECT = "mysql"; //$NON-NLS-1$
  /**
   * <code>DEFAULT_VERSION</code> SQLCommands - 5.0
   */
  private static final String DEFAULT_VERSION = "5.0"; //$NON-NLS-1$
  /**
   * <code>COUNTRY</code> SQLCommands - version
   */
  private static final String VERSION = "version"; //$NON-NLS-1$
  /**
   * <code>LANGUAGE</code> SQLCommands - sqlanguage
   */
  private static final String DIALECT = "sqlanguage"; //$NON-NLS-1$
  /**
   * <code>resource</code> SQLCommands - keep generated strings in here to make lookup faster. Can be done because all result strings are static.
   */
  private final Map<String, String> resource = new HashMap<String, String>();
  /**
   * <code>RESOURCE_BUNDLE</code> SQLCommands -
   */
  protected ResourceBundle RESOURCE_BUNDLE;
  static {
    Configure.load();
  }

  /**
   * Constructor
   */
  public SQLCommands() {
    super();
    final String lang = Configure.getProperty(DIALECT, DEFAULT_DIALECT, SQLCommands.class.getName());
    final String ver = Configure.getProperty(VERSION, DEFAULT_VERSION, SQLCommands.class.getName());
    RESOURCE_BUNDLE = ResourceBundle.getBundle(Files.SQL_BUNDLE_NAME, new Locale(lang, ver));
  }

  /**
   * This method returns a string for a given class name and id.
   * 
   * @param c
   * @param id
   * @return localised String
   */
  public String getString(final Class< ? > c, final int id) {
    final String key = c.getSimpleName() + Strings.DOT + id;
    String value = resource.get(key);
    if (value == null) {
      value = Resource.getString(RESOURCE_BUNDLE, c, id, new ArrayList<Object>());
      resource.put(key, value);
    }
    return value;
  }

  /**
   * This method returns a string for a given key.
   * 
   * @param key
   * @return localised String
   */
  public String getString(final String key) {
    String value = resource.get(key);
    if (value == null) {
      value = Resource.getString(RESOURCE_BUNDLE, key);
      resource.put(key, value);
    }
    return value;
  }
}
